=== modified file 'Makefile'
--- Makefile	2006-08-17 13:33:50 +0000
+++ Makefile	2006-09-05 07:37:01 +0000
@@ -73,7 +73,7 @@
 	python tools/win32/ostools.py copytodir tools/win32/bazaar.url win32_bzr.exe
 
 # win32 installer for bzr.exe
-installer: exe copy_docs
+installer: exe copy-docs
 	@echo *** Make windows installer
 	cog.py -d -o tools/win32/bzr.iss tools/win32/bzr.iss.cog
 	iscc /Q tools/win32/bzr.iss

=== modified file 'NEWS'
--- NEWS	2006-09-05 23:44:49 +0000
+++ NEWS	2006-09-08 01:58:47 +0000
@@ -2,6 +2,20 @@
 
   IMPROVEMENTS:
 
+    * Knit files now wait to create their contents until the first data is
+      added. The old code used to create an empty .knit and a .kndx with just
+      the header. However, this caused a lot of extra round trips over sftp.
+      This can change the time for ``bzr push`` to create a new remote branch
+      from 160s down to 100s. This also affects ``bzr commit`` performance when
+      adding new files, ``bzr commit`` on a new kernel-like tree drops from 50s
+      down to 40s (John Arbash Meinel, #44692)
+
+    * When an entire subtree has been deleted, commit will now report that
+      just the top of the subtree has been deleted, rather than reporting
+      all the individual items. (Robert Collins)
+
+    * Commit performs one less XML parse. (Robert Collins)
+
   BUG FIXES:
 
     * Handle boundary="" lines properly to allow access through a Squid proxy.
@@ -18,11 +32,19 @@
     * Handle when LANG is not recognized by python. Emit a warning, but
       just revert to using 'ascii'. (John Arbash Meinel, #35392)
 
+    * Don't use preexec_fn on win32, as it is not supported by subprocess.
+      (John Arbash Meinel)
+
   INTERNALS:
 
+    * TestCaseInTempDir now creates a separate directory for HOME, rather
+      than having HOME set to the same location as the working directory.
+      (John Arbash Meinel)
+
     * run_bzr_subprocess() can take an optional 'env_changes={}' parameter,
-      which will update os.environ inside the spawned child.
-      (John Arbash Meinel)
+      which will update os.environ inside the spawned child. It also can
+      take a 'universal_newlines=True', which helps when checking the output
+      of the command. (John Arbash Meinel)
 
     * Refactor SFTP vendors to allow easier re-use when ssh is used. 
       (Andrew Bennetts)
@@ -38,9 +60,28 @@
       which is deprecated without it spewing to the screen, just by supplying
       the deprecation format string issued for it. (Robert Collins)
 
-    * WorkingTree.last_revision and WorkingTree.pending_merges are deprecated.
-      Please use the get_parent_ids (introduced in 0.10) method instead.
-      (Robert Collins)
+    * Transport.append and Transport.put have been deprecated in favor of
+      .append_bytes, .append_file, .put_bytes, and .put_file. This removes the
+      ambiguity in what type of object the functions take.
+      Transport.non_atomic_put_{bytes,file} has also been added. Which works
+      similarly to Transport.append() except for SFTP, it doesn't have a round
+      trip when opening the file. Also, it provides functionality for creating
+      a parent directory when trying to create a file, rather than raise
+      NoSuchFile and forcing the caller to repeat their request.
+      (John Arbash Meinel)
+
+    * WorkingTree has a new api ``unversion`` which allow the unversioning of
+      entries by their file id. (Robert Collins)
+
+    * WorkingTree.pending_merges is deprecated.  Please use the get_parent_ids
+      (introduced in 0.10) method instead. (Robert Collins)
+
+    * Add a new method ``Tree.revision_tree`` which allows access to cached
+      trees for arbitrary revisions. This allows the in development dirstate
+      tree format to provide access to the callers to cached copies of 
+      inventory data which are cheaper to access than inventories from the
+      repository.
+      (Robert Collins, Martin Pool)
 
 bzr 0.10.0RC1  2006-08-28
   

=== modified file 'bzrlib/benchmarks/bench_bench.py'
--- bzrlib/benchmarks/bench_bench.py	2006-08-15 03:04:06 +0000
+++ bzrlib/benchmarks/bench_bench.py	2006-09-05 21:25:15 +0000
@@ -34,7 +34,7 @@
         # on roberts machine: this originally took:  7372ms/ 7479ms
         # with the LocalTransport._abspath call:     3730ms/ 3778ms
         # with AtomicFile tuning:                    2888ms/ 2926ms
-        # switching to transport.append:             1468ms/ 2849ms
+        # switching to transport.append_bytes:       1468ms/ 2849ms
         self.time(self.make_kernel_like_tree)
 
     def test_02_make_kernel_like_tree(self):

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-08-31 18:18:09 +0000
+++ bzrlib/branch.py	2006-09-05 22:33:10 +0000
@@ -839,9 +839,9 @@
             raise errors.UninitializableFormat(self)
         mutter('creating branch reference in %s', a_bzrdir.transport.base)
         branch_transport = a_bzrdir.get_branch_transport(self)
-        # FIXME rbc 20060209 one j-a-ms encoding branch lands this str() cast is not needed.
-        branch_transport.put('location', StringIO(str(target_branch.bzrdir.root_transport.base)))
-        branch_transport.put('format', StringIO(self.get_format_string()))
+        branch_transport.put_bytes('location',
+            target_branch.bzrdir.root_transport.base)
+        branch_transport.put_bytes('format', self.get_format_string())
         return self.open(a_bzrdir, _found=True)
 
     def __init__(self):

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-08-30 13:20:22 +0000
+++ bzrlib/bzrdir.py	2006-09-05 18:48:28 +0000
@@ -1479,7 +1479,7 @@
         inv = serializer_v4.read_inventory(self.branch.control_files.get('inventory'))
         new_inv_xml = bzrlib.xml5.serializer_v5.write_inventory_to_string(inv)
         # FIXME inventory is a working tree change.
-        self.branch.control_files.put('inventory', new_inv_xml)
+        self.branch.control_files.put('inventory', StringIO(new_inv_xml))
 
     def _write_all_weaves(self):
         controlweaves = WeaveStore(self.bzrdir.transport, prefixed=False)

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-09-05 08:14:23 +0000
+++ bzrlib/commit.py	2006-09-08 00:19:48 +0000
@@ -468,17 +468,18 @@
         """
         specific = self.specific_files
         deleted_ids = []
+        deleted_paths = set()
         for path, ie in self.work_inv.iter_entries():
+            if is_inside_any(deleted_paths, path):
+                # The tree will delete the required ids recursively.
+                continue
             if specific and not is_inside_any(specific, path):
                 continue
             if not self.work_tree.has_filename(path):
+                deleted_paths.add(path)
                 self.reporter.missing(path)
-                deleted_ids.append((path, ie.file_id))
-        if deleted_ids:
-            deleted_ids.sort(reverse=True)
-            for path, file_id in deleted_ids:
-                del self.work_inv[file_id]
-            self.work_tree._write_inventory(self.work_inv)
+                deleted_ids.append(ie.file_id)
+        self.work_tree.unversion(deleted_ids)
 
     def _populate_new_inv(self):
         """Build revision inventory.

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2006-07-28 16:05:23 +0000
+++ bzrlib/conflicts.py	2006-09-06 12:00:01 +0000
@@ -432,13 +432,15 @@
 
     typestring = 'unversioned parent'
 
-    format = 'Conflict adding versioned files to %(path)s.  %(action)s.'
+    format = 'Conflict because %(path)s is not versioned, but has versioned'\
+             ' children.  %(action)s.'
 
 
 class MissingParent(HandledConflict):
     """An attempt to add files to a directory that is not present.
-    Typically, the result of a merge where one tree deleted the directory and
-    the other added a file to it.
+    Typically, the result of a merge where THIS deleted the directory and
+    the OTHER added a file to it.
+    See also: DeletingParent (same situation, reversed THIS and OTHER)
     """
 
     typestring = 'missing parent'
@@ -446,6 +448,17 @@
     format = 'Conflict adding files to %(path)s.  %(action)s.'
 
 
+class DeletingParent(HandledConflict):
+    """An attempt to add files to a directory that is not present.
+    Typically, the result of a merge where one OTHER deleted the directory and
+    the THIS added a file to it.
+    """
+
+    typestring = 'deleting parent'
+
+    format = "Conflict: can't delete %(path)s because it is not empty.  "\
+             "%(action)s."
+
 
 ctype = {}
 
@@ -458,4 +471,5 @@
 
 
 register_types(ContentsConflict, TextConflict, PathConflict, DuplicateID,
-               DuplicateEntry, ParentLoop, UnversionedParent, MissingParent,)
+               DuplicateEntry, ParentLoop, UnversionedParent, MissingParent,
+               DeletingParent,)

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-08-25 20:43:30 +0000
+++ bzrlib/diff.py	2006-09-06 15:32:11 +0000
@@ -103,7 +103,12 @@
     :return: A Popen object.
     """
     if capture_errors:
-        preexec_fn = _set_lang_C
+        if sys.platform == 'win32':
+            # Win32 doesn't support preexec_fn, but that is
+            # okay, because it doesn't support LANG either.
+            preexec_fn = None
+        else:
+            preexec_fn = _set_lang_C
         stderr = subprocess.PIPE
     else:
         preexec_fn = None

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-09-04 08:00:48 +0000
+++ bzrlib/errors.py	2006-09-08 01:58:47 +0000
@@ -174,6 +174,15 @@
         self.branch = branch
 
 
+class NoSuchId(BzrNewError):
+    """The file id %(file_id)s is not present in the tree %(tree)s."""
+    
+    def __init__(self, tree, file_id):
+        BzrNewError.__init__(self)
+        self.file_id = file_id
+        self.tree = tree
+
+
 class NoWorkingTree(BzrNewError):
     """No WorkingTree exists for %(base)s."""
     
@@ -518,6 +527,15 @@
         BzrNewError.__init__(self, branch=branch, revision=revision)
 
 
+class NoSuchRevisionInTree(NoSuchRevision):
+    """The revision id %(revision_id)s is not present in the tree %(tree)s."""
+
+    def __init__(self, tree, revision_id):
+        BzrNewError.__init__(self)
+        self.tree = tree
+        self.revision_id = revision_id
+
+
 class NoSuchRevisionSpec(BzrNewError):
     """No namespace registered for string: %(spec)r"""
 

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2006-08-25 14:07:28 +0000
+++ bzrlib/gpg.py	2006-09-07 20:20:17 +0000
@@ -20,6 +20,7 @@
 import errno
 import os
 import subprocess
+import sys
 
 from bzrlib import (
     errors,
@@ -72,11 +73,16 @@
 
     def sign(self, content):
         ui.ui_factory.clear_term()
+
+        preexec_fn = _set_gpg_tty
+        if sys.platform == 'win32':
+            # Win32 doesn't support preexec_fn, but wouldn't support TTY anyway.
+            preexec_fn = None
         try:
             process = subprocess.Popen(self._command_line(),
                                        stdin=subprocess.PIPE,
                                        stdout=subprocess.PIPE,
-                                       preexec_fn=_set_gpg_tty)
+                                       preexec_fn=preexec_fn)
             try:
                 result = process.communicate(content)[0]
                 if process.returncode is None:

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-08-30 13:54:51 +0000
+++ bzrlib/inventory.py	2006-09-07 22:28:25 +0000
@@ -1217,6 +1217,24 @@
     def has_id(self, file_id):
         return (file_id in self._byid)
 
+    def remove_recursive_id(self, file_id):
+        """Remove file_id, and children, from the inventory.
+        
+        :param file_id: A file_id to remove.
+        """
+        to_find_delete = [self._byid[file_id]]
+        to_delete = []
+        while to_find_delete:
+            ie = to_find_delete.pop()
+            to_delete.append(ie.file_id)
+            if ie.kind == 'directory':
+                to_find_delete.extend(ie.children.values())
+        for file_id in reversed(to_delete):
+            ie = self[file_id]
+            del self._byid[file_id]
+            if ie.parent_id is not None:
+                del self[ie.parent_id].children[ie.name]
+
     def rename(self, file_id, new_parent_id, new_name):
         """Move a file within the inventory.
 

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-08-30 13:54:51 +0000
+++ bzrlib/knit.py	2006-09-06 21:56:58 +0000
@@ -279,12 +279,18 @@
     stored and retrieved.
     """
 
-    def __init__(self, relpath, transport, file_mode=None, access_mode=None, 
+    def __init__(self, relpath, transport, file_mode=None, access_mode=None,
                  factory=None, basis_knit=DEPRECATED_PARAMETER, delta=True,
-                 create=False):
+                 create=False, create_parent_dir=False, delay_create=False,
+                 dir_mode=None):
         """Construct a knit at location specified by relpath.
         
         :param create: If not True, only open an existing knit.
+        :param create_parent_dir: If True, create the parent directory if 
+            creating the file fails. (This is used for stores with 
+            hash-prefixes that may not exist yet)
+        :param delay_create: The calling code is aware that the knit won't 
+            actually be created until the first data is stored.
         """
         if deprecated_passed(basis_knit):
             warnings.warn("KnitVersionedFile.__(): The basis_knit parameter is"
@@ -301,9 +307,13 @@
         self.delta = delta
 
         self._index = _KnitIndex(transport, relpath + INDEX_SUFFIX,
-            access_mode, create=create, file_mode=file_mode)
+            access_mode, create=create, file_mode=file_mode,
+            create_parent_dir=create_parent_dir, delay_create=delay_create,
+            dir_mode=dir_mode)
         self._data = _KnitData(transport, relpath + DATA_SUFFIX,
-            access_mode, create=create and not len(self), file_mode=file_mode)
+            access_mode, create=create and not len(self), file_mode=file_mode,
+            create_parent_dir=create_parent_dir, delay_create=delay_create,
+            dir_mode=dir_mode)
 
     def __repr__(self):
         return '%s(%s)' % (self.__class__.__name__, 
@@ -403,18 +413,20 @@
         """See VersionedFile.copy_to()."""
         # copy the current index to a temp index to avoid racing with local
         # writes
-        transport.put(name + INDEX_SUFFIX + '.tmp', self.transport.get(self._index._filename),)
+        transport.put_file_non_atomic(name + INDEX_SUFFIX + '.tmp',
+                self.transport.get(self._index._filename))
         # copy the data file
         f = self._data._open_file()
         try:
-            transport.put(name + DATA_SUFFIX, f)
+            transport.put_file(name + DATA_SUFFIX, f)
         finally:
             f.close()
         # move the copied index into place
         transport.move(name + INDEX_SUFFIX + '.tmp', name + INDEX_SUFFIX)
 
     def create_empty(self, name, transport, mode=None):
-        return KnitVersionedFile(name, transport, factory=self.factory, delta=self.delta, create=True)
+        return KnitVersionedFile(name, transport, factory=self.factory,
+                                 delta=self.delta, create=True)
     
     def _fix_parents(self, version, new_parents):
         """Fix the parents list for version.
@@ -946,16 +958,15 @@
 class _KnitComponentFile(object):
     """One of the files used to implement a knit database"""
 
-    def __init__(self, transport, filename, mode, file_mode=None):
+    def __init__(self, transport, filename, mode, file_mode=None,
+                 create_parent_dir=False, dir_mode=None):
         self._transport = transport
         self._filename = filename
         self._mode = mode
-        self._file_mode=file_mode
-
-    def write_header(self):
-        if self._transport.append(self._filename, StringIO(self.HEADER),
-            mode=self._file_mode):
-            raise KnitCorrupt(self._filename, 'misaligned after writing header')
+        self._file_mode = file_mode
+        self._dir_mode = dir_mode
+        self._create_parent_dir = create_parent_dir
+        self._need_to_create = False
 
     def check_header(self, fp):
         line = fp.readline()
@@ -1048,8 +1059,12 @@
                                    parents,
                                    index)
 
-    def __init__(self, transport, filename, mode, create=False, file_mode=None):
-        _KnitComponentFile.__init__(self, transport, filename, mode, file_mode)
+    def __init__(self, transport, filename, mode, create=False, file_mode=None,
+                 create_parent_dir=False, delay_create=False, dir_mode=None):
+        _KnitComponentFile.__init__(self, transport, filename, mode,
+                                    file_mode=file_mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
         self._cache = {}
         # position in _history is the 'official' index for a revision
         # but the values may have come from a newer entry.
@@ -1124,7 +1139,12 @@
             except NoSuchFile, e:
                 if mode != 'w' or not create:
                     raise
-                self.write_header()
+                if delay_create:
+                    self._need_to_create = True
+                else:
+                    self._transport.put_bytes_non_atomic(self._filename,
+                        self.HEADER, mode=self._file_mode)
+
         finally:
             pb.update('read knit index', total, total)
             pb.finished()
@@ -1245,7 +1265,19 @@
             assert isinstance(line, str), \
                 'content must be utf-8 encoded: %r' % (line,)
             lines.append(line)
-        self._transport.append(self._filename, StringIO(''.join(lines)))
+        if not self._need_to_create:
+            self._transport.append_bytes(self._filename, ''.join(lines))
+        else:
+            sio = StringIO()
+            sio.write(self.HEADER)
+            sio.writelines(lines)
+            sio.seek(0)
+            self._transport.put_file_non_atomic(self._filename, sio,
+                                create_parent_dir=self._create_parent_dir,
+                                mode=self._file_mode,
+                                dir_mode=self._dir_mode)
+            self._need_to_create = False
+
         # cache after writing, so that a failed write leads to missing cache
         # entries not extra ones. XXX TODO: RBC 20060502 in the event of a 
         # failure, reload the index or flush it or some such, to prevent
@@ -1296,10 +1328,13 @@
 class _KnitData(_KnitComponentFile):
     """Contents of the knit data file"""
 
-    HEADER = "# bzr knit data 8\n"
-
-    def __init__(self, transport, filename, mode, create=False, file_mode=None):
-        _KnitComponentFile.__init__(self, transport, filename, mode)
+    def __init__(self, transport, filename, mode, create=False, file_mode=None,
+                 create_parent_dir=False, delay_create=False,
+                 dir_mode=None):
+        _KnitComponentFile.__init__(self, transport, filename, mode,
+                                    file_mode=file_mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
         self._checked = False
         # TODO: jam 20060713 conceptually, this could spill to disk
         #       if the cached size gets larger than a certain amount
@@ -1308,7 +1343,11 @@
         self._cache = {}
         self._do_cache = False
         if create:
-            self._transport.put(self._filename, StringIO(''), mode=file_mode)
+            if delay_create:
+                self._need_to_create = create
+            else:
+                self._transport.put_bytes_non_atomic(self._filename, '',
+                                                     mode=self._file_mode)
 
     def enable_cache(self):
         """Enable caching of reads."""
@@ -1353,14 +1392,30 @@
         :return: the offset in the data file raw_data was written.
         """
         assert isinstance(raw_data, str), 'data must be plain bytes'
-        return self._transport.append(self._filename, StringIO(raw_data))
+        if not self._need_to_create:
+            return self._transport.append_bytes(self._filename, raw_data)
+        else:
+            self._transport.put_bytes_non_atomic(self._filename, raw_data,
+                                   create_parent_dir=self._create_parent_dir,
+                                   mode=self._file_mode,
+                                   dir_mode=self._dir_mode)
+            self._need_to_create = False
+            return 0
         
     def add_record(self, version_id, digest, lines):
         """Write new text record to disk.  Returns the position in the
         file where it was written."""
         size, sio = self._record_to_data(version_id, digest, lines)
         # write to disk
-        start_pos = self._transport.append(self._filename, sio)
+        if not self._need_to_create:
+            start_pos = self._transport.append_file(self._filename, sio)
+        else:
+            self._transport.put_file_non_atomic(self._filename, sio,
+                               create_parent_dir=self._create_parent_dir,
+                               mode=self._file_mode,
+                               dir_mode=self._dir_mode)
+            self._need_to_create = False
+            start_pos = 0
         if self._do_cache:
             self._cache[version_id] = sio.getvalue()
         return start_pos, size

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2006-07-28 16:05:23 +0000
+++ bzrlib/lockable_files.py	2006-09-05 19:58:39 +0000
@@ -193,7 +193,7 @@
                      directory
         :param f: A file-like or string object whose contents should be copied.
         """
-        self._transport.put(self._escape(path), file, mode=self._file_mode)
+        self._transport.put_file(self._escape(path), file, mode=self._file_mode)
 
     @needs_write_lock
     def put_utf8(self, path, a_string):
@@ -339,5 +339,5 @@
     def create(self, mode=None):
         """Create lock mechanism"""
         # for old-style locks, create the file now
-        self._transport.put(self._escaped_name, StringIO(), 
+        self._transport.put_bytes(self._escaped_name, '',
                             mode=self._file_modebits)

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2006-08-24 20:04:13 +0000
+++ bzrlib/lockdir.py	2006-09-05 21:25:15 +0000
@@ -207,7 +207,7 @@
             # because we don't want to write to a temporary file and rename
             # into place, because that's going to happen to the whole
             # directory
-            self.transport.append(tmpname + self.__INFO_NAME, sio)
+            self.transport.append_file(tmpname + self.__INFO_NAME, sio)
 
             self.transport.rename(tmpname, self._held_dir)
             self._lock_held = True

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-08-25 15:05:57 +0000
+++ bzrlib/osutils.py	2006-09-05 13:57:30 +0000
@@ -867,10 +867,30 @@
 
     return width
 
+
 def supports_executable():
     return sys.platform != "win32"
 
 
+def set_or_unset_env(env_variable, value):
+    """Modify the environment, setting or removing the env_variable.
+
+    :param env_variable: The environment variable in question
+    :param value: The value to set the environment to. If None, then
+        the variable will be removed.
+    :return: The original value of the environment variable.
+    """
+    orig_val = os.environ.get(env_variable)
+    if value is None:
+        if orig_val is not None:
+            del os.environ[env_variable]
+    else:
+        if isinstance(value, unicode):
+            value = value.encode(bzrlib.user_encoding)
+        os.environ[env_variable] = value
+    return orig_val
+
+
 _validWin32PathRE = re.compile(r'^([A-Za-z]:[/\\])?[^:<>*"?\|]*$')
 
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-08-16 21:10:33 +0000
+++ bzrlib/repository.py	2006-09-06 21:49:40 +0000
@@ -1088,6 +1088,7 @@
                                   control_files,
                                   prefixed=True,
                                   versionedfile_class=WeaveFile,
+                                  versionedfile_kwargs={},
                                   escaped=False):
         weave_transport = control_files._transport.clone(name)
         dir_mode = control_files._dir_mode
@@ -1096,6 +1097,7 @@
                                   dir_mode=dir_mode,
                                   file_mode=file_mode,
                                   versionedfile_class=versionedfile_class,
+                                  versionedfile_kwargs=versionedfile_kwargs,
                                   escaped=escaped)
 
     def initialize(self, a_bzrdir, shared=False):
@@ -1490,7 +1492,7 @@
             prefixed=False,
             precious=True,
             versionedfile_class=KnitVersionedFile,
-            versionedfile_kwargs={'delta':False, 'factory':KnitPlainFactory()},
+            versionedfile_kwargs={'delta':False, 'factory':KnitPlainFactory(),},
             escaped=True,
             )
         return KnitRevisionStore(versioned_file_store)
@@ -1501,6 +1503,11 @@
                                               transport,
                                               control_files,
                                               versionedfile_class=KnitVersionedFile,
+                                              versionedfile_kwargs={
+                                                  'create_parent_dir':True,
+                                                  'delay_create':True,
+                                                  'dir_mode':control_files._dir_mode,
+                                              },
                                               escaped=True)
 
     def initialize(self, a_bzrdir, shared=False):

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2006-07-30 07:02:22 +0000
+++ bzrlib/store/__init__.py	2006-09-05 20:58:28 +0000
@@ -30,13 +30,20 @@
 from zlib import adler32
 
 import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    symbol_versioning,
+    urlutils,
+    )
 from bzrlib.errors import BzrError, UnlistableStore, TransportNotPossible
-from bzrlib.symbol_versioning import (deprecated_function, zero_eight)
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    zero_eight,
+    zero_eleven,
+    )
 from bzrlib.trace import mutter
 from bzrlib.transport import Transport
 from bzrlib.transport.local import LocalTransport
-import bzrlib.urlutils as urlutils
 
 ######################################################################
 # stores
@@ -164,9 +171,13 @@
     def add(self, f, fileid, suffix=None):
         """Add contents of a file into the store.
 
-        f -- A file-like object, or string
+        f -- A file-like object
         """
         mutter("add store entry %r", fileid)
+        if isinstance(f, str):
+            symbol_versioning.warn(zero_eleven % 'Passing a string to Store.add',
+                DeprecationWarning, stacklevel=2)
+            f = StringIO(f)
         
         names = self._id_to_names(fileid, suffix)
         if self._transport.has_any(names):
@@ -178,7 +189,6 @@
         # doesn't exist), then create the dir, and try again
         self._add(names[0], f)
 
-
     def _add(self, relpath, f):
         """Actually add the file to the given location.
         This should be overridden by children.

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2006-07-28 16:05:23 +0000
+++ bzrlib/store/text.py	2006-09-05 20:08:52 +0000
@@ -63,7 +63,7 @@
 
     def _try_put(self, fn, f):
         try:
-            self._transport.put(fn, f, mode=self._file_mode)
+            self._transport.put_file(fn, f, mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
@@ -71,7 +71,7 @@
                 self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
             except FileExists:
                 pass
-            self._transport.put(fn, f, mode=self._file_mode)
+            self._transport.put_file(fn, f, mode=self._file_mode)
 
     def _get(self, fn):
         if fn.endswith('.gz'):

=== modified file 'bzrlib/store/versioned/__init__.py'
--- bzrlib/store/versioned/__init__.py	2006-07-28 16:05:23 +0000
+++ bzrlib/store/versioned/__init__.py	2006-09-05 20:08:52 +0000
@@ -113,12 +113,12 @@
     def _put(self, file_id, f):
         fn = self.filename(file_id)
         try:
-            return self._transport.put(fn, f, mode=self._file_mode)
+            return self._transport.put_file(fn, f, mode=self._file_mode)
         except errors.NoSuchFile:
             if not self._prefixed:
                 raise
             self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
-            return self._transport.put(fn, f, mode=self._file_mode)
+            return self._transport.put_file(fn, f, mode=self._file_mode)
 
     def get_weave(self, file_id, transaction, _filename=None):
         """Return the VersionedFile for file_id.

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-09-05 23:44:49 +0000
+++ bzrlib/tests/__init__.py	2006-09-08 00:19:48 +0000
@@ -689,33 +689,20 @@
             'BZR_EMAIL': None,
             'BZREMAIL': None, # may still be present in the environment
             'EMAIL': None,
+            'BZR_PROGRESS_BAR': None,
         }
         self.__old_env = {}
         self.addCleanup(self._restoreEnvironment)
         for name, value in new_env.iteritems():
             self._captureVar(name, value)
 
-
     def _captureVar(self, name, newvalue):
-        """Set an environment variable, preparing it to be reset when finished."""
-        self.__old_env[name] = os.environ.get(name, None)
-        if newvalue is None:
-            if name in os.environ:
-                del os.environ[name]
-        else:
-            os.environ[name] = newvalue
-
-    @staticmethod
-    def _restoreVar(name, value):
-        if value is None:
-            if name in os.environ:
-                del os.environ[name]
-        else:
-            os.environ[name] = value
+        """Set an environment variable, and reset it when finished."""
+        self.__old_env[name] = osutils.set_or_unset_env(name, newvalue)
 
     def _restoreEnvironment(self):
         for name, value in self.__old_env.iteritems():
-            self._restoreVar(name, value)
+            osutils.set_or_unset_env(name, value)
 
     def tearDown(self):
         self._runCleanups()
@@ -896,22 +883,40 @@
             variables. A value of None will unset the env variable.
             The values must be strings. The change will only occur in the
             child, so you don't need to fix the environment after running.
+        :param universal_newlines: Convert CRLF => LF
         """
         env_changes = kwargs.get('env_changes', {})
+
+        old_env = {}
+
         def cleanup_environment():
             for env_var, value in env_changes.iteritems():
-                if value is None:
-                    del os.environ[env_var]
-                else:
-                    os.environ[env_var] = value
+                old_env[env_var] = osutils.set_or_unset_env(env_var, value)
+
+        def restore_environment():
+            for env_var, value in old_env.iteritems():
+                osutils.set_or_unset_env(env_var, value)
 
         bzr_path = os.path.dirname(os.path.dirname(bzrlib.__file__))+'/bzr'
         args = list(args)
-        process = Popen([sys.executable, bzr_path]+args,
-                         stdout=PIPE, stderr=PIPE,
-                         preexec_fn=cleanup_environment)
+
+        try:
+            # win32 subprocess doesn't support preexec_fn
+            # so we will avoid using it on all platforms, just to
+            # make sure the code path is used, and we don't break on win32
+            cleanup_environment()
+            process = Popen([sys.executable, bzr_path]+args,
+                             stdout=PIPE, stderr=PIPE)
+        finally:
+            restore_environment()
+            
         out = process.stdout.read()
         err = process.stderr.read()
+
+        if kwargs.get('universal_newlines', False):
+            out = out.replace('\r\n', '\n')
+            err = err.replace('\r\n', '\n')
+
         retcode = process.wait()
         supplied_retcode = kwargs.get('retcode', 0)
         if supplied_retcode is not None:
@@ -1056,12 +1061,15 @@
                 i = i + 1
                 continue
             else:
-                self.test_dir = candidate_dir
+                os.mkdir(candidate_dir)
+                self.test_home_dir = candidate_dir + '/home'
+                os.mkdir(self.test_home_dir)
+                self.test_dir = candidate_dir + '/work'
                 os.mkdir(self.test_dir)
                 os.chdir(self.test_dir)
                 break
-        os.environ['HOME'] = self.test_dir
-        os.environ['APPDATA'] = self.test_dir
+        os.environ['HOME'] = self.test_home_dir
+        os.environ['APPDATA'] = self.test_home_dir
         def _leaveDirectory():
             os.chdir(_currentdir)
         self.addCleanup(_leaveDirectory)
@@ -1106,7 +1114,8 @@
                 # On jam's machine, make_kernel_like_tree is:
                 #   put:    4.5-7.5s (averaging 6s)
                 #   append: 2.9-4.5s
-                transport.append(urlutils.escape(name), StringIO(content))
+                #   put_non_atomic: 2.9-4.5s
+                transport.put_bytes_non_atomic(urlutils.escape(name), content)
 
     def build_tree_contents(self, shape):
         build_tree_contents(shape)

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2006-08-16 08:03:47 +0000
+++ bzrlib/tests/blackbox/test_add.py	2006-09-07 21:31:58 +0000
@@ -19,8 +19,6 @@
 
 import os
 
-from bzrlib import ignores
-
 from bzrlib.tests.blackbox import ExternalBase
 
 
@@ -28,8 +26,6 @@
         
     def test_add_reports(self):
         """add command prints the names of added files."""
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.runbzr('init')
         self.build_tree(['top.txt', 'dir/', 'dir/sub.txt', 'CVS'])
         self.build_tree_contents([('.bzrignore', 'CVS\n')])
@@ -42,13 +38,12 @@
                            'added dir',
                            'added dir/sub.txt',
                            'added top.txt',
-                           'ignored 2 file(s).'],
+                           'ignored 1 file(s).'],
                           results)
         out = self.run_bzr_captured(['add', '-v'], retcode=0)[0]
         results = sorted(out.rstrip('\n').split('\n'))
         self.assertEquals(['If you wish to add some of these files, please'\
                            ' add them by name.',
-                           'ignored .bazaar matching "./.bazaar"',
                            'ignored CVS matching "CVS"'],
                           results)
 
@@ -66,8 +61,6 @@
 
         "bzr add" should add the parent(s) as necessary.
         """
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.runbzr('init')
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
@@ -91,8 +84,6 @@
 
         "bzr add" should do this happily.
         """
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.runbzr('init')
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
@@ -105,7 +96,6 @@
         """Add in subdirectory should add only things from there down"""
         from bzrlib.workingtree import WorkingTree
 
-        ignores._set_user_ignores(['./.bazaar'])
         eq = self.assertEqual
         ass = self.assertTrue
         chdir = os.chdir

=== modified file 'bzrlib/tests/blackbox/test_ignore.py'
--- bzrlib/tests/blackbox/test_ignore.py	2006-07-21 04:02:14 +0000
+++ bzrlib/tests/blackbox/test_ignore.py	2006-09-07 21:37:43 +0000
@@ -47,7 +47,7 @@
 
         # is_ignored() will now create the user global ignore file
         # if it doesn't exist, so make sure we ignore it in our tests
-        ignores._set_user_ignores(['*.tmp', './.bazaar'])
+        ignores._set_user_ignores(['*.tmp'])
 
         self.build_tree_contents(
             [('foo.tmp', '.tmp files are ignored by default'),

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2006-08-16 04:57:05 +0000
+++ bzrlib/tests/blackbox/test_init.py	2006-09-05 11:22:35 +0000
@@ -73,7 +73,7 @@
         self.assertEqual('', out)
         self.assertContainsRe(err,
             r'^bzr: ERROR: .*'
-            '\[Errno 2\] No such file or directory: ')
+            '\[Errno 2\] No such file or directory')
         
         os.mkdir('subdir2')
         out, err = self.run_bzr('init', 'subdir2')

=== modified file 'bzrlib/tests/blackbox/test_locale.py'
--- bzrlib/tests/blackbox/test_locale.py	2006-08-30 13:38:22 +0000
+++ bzrlib/tests/blackbox/test_locale.py	2006-09-05 11:24:56 +0000
@@ -19,7 +19,7 @@
 import os
 import sys
 
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import TestCaseWithTransport, TestSkipped
 
 
 class TestLocale(TestCaseWithTransport):

=== modified file 'bzrlib/tests/blackbox/test_ls.py'
--- bzrlib/tests/blackbox/test_ls.py	2006-07-21 04:02:14 +0000
+++ bzrlib/tests/blackbox/test_ls.py	2006-09-07 21:31:58 +0000
@@ -28,7 +28,7 @@
         super(TestLS, self).setUp()
         
         # Create a simple branch that can be used in testing
-        ignores._set_user_ignores(['./.bazaar', 'user-ignore'])
+        ignores._set_user_ignores(['user-ignore'])
 
         self.wt = self.make_branch_and_tree('.')
         self.build_tree_contents([
@@ -48,41 +48,36 @@
 
     def test_ls_basic(self):
         """Test the abilities of 'bzr ls'"""
-        self.ls_equals('.bazaar\n.bzrignore\na\n')
-        self.ls_equals('I        .bazaar/\n'
-                       '?        .bzrignore\n'
+        self.ls_equals('.bzrignore\na\n')
+        self.ls_equals('?        .bzrignore\n'
                        '?        a\n',
                        '--verbose')
         self.ls_equals('.bzrignore\n'
                        'a\n',
                        '--unknown')
-        self.ls_equals('.bazaar\n', '--ignored')
+        self.ls_equals('', '--ignored')
         self.ls_equals('', '--versioned')
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n',
                        '--unknown', '--ignored', '--versioned')
-        self.ls_equals('.bazaar\n', '--ignored', '--versioned')
-        self.ls_equals('.bazaar\0.bzrignore\0a\0', '--null')
+        self.ls_equals('', '--ignored', '--versioned')
+        self.ls_equals('.bzrignore\0a\0', '--null')
 
     def test_ls_added(self):
         self.wt.add(['a'])
-        self.ls_equals('I        .bazaar/\n'
-                       '?        .bzrignore\n'
+        self.ls_equals('?        .bzrignore\n'
                        'V        a\n',
                        '--verbose')
         self.wt.commit('add')
         
         self.build_tree(['subdir/'])
-        self.ls_equals('I        .bazaar/\n'
-                       '?        .bzrignore\n'
+        self.ls_equals('?        .bzrignore\n'
                        'V        a\n'
                        '?        subdir/\n'
                        , '--verbose')
         self.build_tree(['subdir/b'])
         self.wt.add(['subdir/', 'subdir/b', '.bzrignore'])
-        self.ls_equals('I        .bazaar/\n'
-                       'V        .bzrignore\n'
+        self.ls_equals('V        .bzrignore\n'
                        'V        a\n'
                        'V        subdir/\n'
                        'V        subdir/b\n'
@@ -92,14 +87,12 @@
         self.build_tree(['subdir/', 'subdir/b'])
         self.wt.add(['a', 'subdir/', 'subdir/b', '.bzrignore'])
 
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n'
                        'subdir\n'
                        , '--non-recursive')
 
-        self.ls_equals('I        .bazaar/\n'
-                       'V        .bzrignore\n'
+        self.ls_equals('V        .bzrignore\n'
                        'V        a\n'
                        'V        subdir/\n'
                        , '--verbose', '--non-recursive')
@@ -109,20 +102,17 @@
         self.ls_equals('b\n')
         self.ls_equals('b\0'
                   , '--null')
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n'
                        'subdir\n'
                        'subdir/b\n'
                        , '--from-root')
-        self.ls_equals('.bazaar\0'
-                       '.bzrignore\0'
+        self.ls_equals('.bzrignore\0'
                        'a\0'
                        'subdir\0'
                        'subdir/b\0'
                        , '--from-root', '--null')
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n'
                        'subdir\n'
                        , '--from-root', '--non-recursive')
@@ -146,22 +136,19 @@
         self.wt.add(['a', '.bzrignore'])
 
         self.build_tree(['blah.py', 'blah.pyo', 'user-ignore'])
-        self.ls_equals('.bazaar\n'
-                       '.bzrignore\n'
+        self.ls_equals('.bzrignore\n'
                        'a\n'
                        'blah.py\n'
                        'blah.pyo\n'
                        'user-ignore\n'
                        )
-        self.ls_equals('I        .bazaar/\n'
-                       'V        .bzrignore\n'
+        self.ls_equals('V        .bzrignore\n'
                        'V        a\n'
                        '?        blah.py\n'
                        'I        blah.pyo\n'
                        'I        user-ignore\n'
                        , '--verbose')
-        self.ls_equals('.bazaar\n'
-                       'blah.pyo\n'
+        self.ls_equals('blah.pyo\n'
                        'user-ignore\n'
                        , '--ignored')
         self.ls_equals('blah.py\n'

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2006-08-06 23:35:09 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2006-09-07 21:37:43 +0000
@@ -20,7 +20,7 @@
 import sys
 import os
 
-from bzrlib import ignores, osutils, urlutils
+from bzrlib import osutils, urlutils
 import bzrlib
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.trace import mutter, note
@@ -471,7 +471,6 @@
     def test_ignore(self):
         bzr = self.run_bzr_decode
 
-        ignores._set_user_ignores(['./.bazaar'])
         fname2 = self.info['filename'] + '2.txt'
         open(fname2, 'wb').write('ignored\n')
 

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2006-08-30 13:38:22 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2006-09-07 20:29:07 +0000
@@ -185,7 +185,7 @@
         self.assertEqual(None, os.environ.get('BZR_EMAIL'))
         out, err = self.run_bzr_subprocess('whoami', env_changes={
                                             'BZR_EMAIL':'Joe Foo <joe@foo.com>'
-                                          })
+                                          }, universal_newlines=True)
         self.assertEqual('', err)
         self.assertEqual('Joe Foo <joe@foo.com>\n', out)
         # And it should not be modified
@@ -195,7 +195,7 @@
         # it is actually changing
         out, err = self.run_bzr_subprocess('whoami', env_changes={
                                             'BZR_EMAIL':'Barry <bar@foo.com>'
-                                          })
+                                          }, universal_newlines=True)
         self.assertEqual('', err)
         self.assertEqual('Barry <bar@foo.com>\n', out)
         self.assertEqual(None, os.environ.get('BZR_EMAIL'))
@@ -209,20 +209,22 @@
         os.environ['EMAIL'] = rand_email
         try:
             # By default, the child will inherit the current env setting
-            out, err = self.run_bzr_subprocess('whoami')
+            out, err = self.run_bzr_subprocess('whoami', universal_newlines=True)
             self.assertEqual('', err)
             self.assertEqual(rand_bzr_email + '\n', out)
 
             # Now that BZR_EMAIL is not set, it should fall back to EMAIL
             out, err = self.run_bzr_subprocess('whoami',
-                                               env_changes={'BZR_EMAIL':None})
+                                               env_changes={'BZR_EMAIL':None},
+                                               universal_newlines=True)
             self.assertEqual('', err)
             self.assertEqual(rand_email + '\n', out)
 
             # This switches back to the default email guessing logic
             # Which shouldn't match either of the above addresses
             out, err = self.run_bzr_subprocess('whoami',
-                           env_changes={'BZR_EMAIL':None, 'EMAIL':None})
+                           env_changes={'BZR_EMAIL':None, 'EMAIL':None},
+                           universal_newlines=True)
 
             self.assertEqual('', err)
             self.assertNotEqual(rand_bzr_email + '\n', out)
@@ -232,6 +234,16 @@
             del os.environ['BZR_EMAIL']
             del os.environ['EMAIL']
 
+    def test_run_bzr_subprocess_env_del_missing(self):
+        """run_bzr_subprocess won't fail if deleting a nonexistant env var"""
+        self.failIf('NON_EXISTANT_ENV_VAR' in os.environ)
+        out, err = self.run_bzr_subprocess('rocks',
+                        env_changes={'NON_EXISTANT_ENV_VAR':None},
+                        universal_newlines=True)
+        self.assertEqual('it sure does!\n', out)
+        self.assertEqual('', err)
+
+
 class TestRunBzrError(ExternalBase):
 
     def test_run_bzr_error(self):

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2006-08-31 18:18:09 +0000
+++ bzrlib/tests/blackbox/test_status.py	2006-09-07 21:37:43 +0000
@@ -28,7 +28,7 @@
 import sys
 from tempfile import TemporaryFile
 
-from bzrlib import bzrdir, errors, ignores
+from bzrlib import bzrdir, errors
 import bzrlib.branch
 from bzrlib.builtins import merge
 from bzrlib.osutils import pathjoin
@@ -62,8 +62,6 @@
         """Test basic branch status"""
         wt = self.make_branch_and_tree('.')
 
-        ignores._set_user_ignores(['./.bazaar'])
-
         # status with no commits or files - it must
         # work and show no output. We do this with no
         # commits to be sure that it's not going to fail
@@ -95,8 +93,6 @@
         """Tests branch status with revisions"""
         wt = self.make_branch_and_tree('.')
 
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.build_tree(['hello.c', 'bye.c'])
         wt.add('hello.c')
         wt.add('bye.c')
@@ -153,8 +149,6 @@
         wt = self.make_branch_and_tree('.')
         b = wt.branch
 
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.build_tree(['directory/','directory/hello.c', 'bye.c','test.c','dir2/'])
         wt.add('directory')
         wt.add('test.c')
@@ -215,8 +209,6 @@
 class TestStatus(TestCaseWithTransport):
 
     def test_status(self):
-        ignores._set_user_ignores(['./.bazaar'])
-
         self.run_bzr("init")
         self.build_tree(['hello.txt'])
         result = self.run_bzr("status")[0]

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/stub_sftp.py	2006-09-06 21:45:49 +0000
@@ -158,7 +158,9 @@
             if getattr(attr, 'st_mode', None):
                 fd = os.open(path, flags, attr.st_mode)
             else:
-                fd = os.open(path, flags)
+                # os.open() defaults to 0777 which is
+                # an odd default mode for files
+                fd = os.open(path, flags, 0666)
         except OSError, e:
             return SFTPServer.convert_errno(e.errno)
 

=== modified file 'bzrlib/tests/test_atomicfile.py'
--- bzrlib/tests/test_atomicfile.py	2006-08-28 19:11:56 +0000
+++ bzrlib/tests/test_atomicfile.py	2006-09-05 10:38:46 +0000
@@ -26,7 +26,7 @@
     osutils,
     symbol_versioning,
     )
-from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
 
 
 class TestAtomicFile(TestCaseInTempDir):
@@ -86,7 +86,7 @@
 
     def _test_mode(self, mode):
         if not self.can_sys_preserve_mode():
-            return
+            raise TestSkipped("This test cannot be run on your platform")
         f = atomicfile.AtomicFile('test', mode='wb', new_mode=mode)
         f.write('foo\n')
         f.commit()

=== modified file 'bzrlib/tests/test_bad_files.py'
--- bzrlib/tests/test_bad_files.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_bad_files.py	2006-09-07 21:37:43 +0000
@@ -18,7 +18,6 @@
 """Tests being able to ignore mad filetypes.
 """
 
-from bzrlib import ignores
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.errors import BadFileKindError
 import os
@@ -63,9 +62,6 @@
             # or PlatformDeficiency
             return
 
-        # Ignore the .bazaar/ home directory that is created
-        ignores._set_user_ignores(['./.bazaar'])
-
         # status with nothing
         verify_status(self, wt, [])
 

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2006-04-28 17:18:42 +0000
+++ bzrlib/tests/test_branch.py	2006-09-05 20:08:52 +0000
@@ -128,7 +128,7 @@
     def initialize(self, a_bzrdir):
         """Format 4 branches cannot be created."""
         t = a_bzrdir.get_branch_transport(self)
-        t.put('format', StringIO(self.get_format_string()))
+        t.put_bytes('format', self.get_format_string())
         return 'A branch'
 
     def is_supported(self):

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2006-07-05 21:03:58 +0000
+++ bzrlib/tests/test_bzrdir.py	2006-09-05 20:58:28 +0000
@@ -94,7 +94,7 @@
         """Create a bzr dir."""
         t = get_transport(url)
         t.mkdir('.bzr')
-        t.put('.bzr/branch-format', StringIO(self.get_format_string()))
+        t.put_bytes('.bzr/branch-format', self.get_format_string())
         return SampleBzrDir(t, self)
 
     def is_supported(self):
@@ -129,7 +129,7 @@
     def test_find_format_unknown_format(self):
         t = get_transport(self.get_url())
         t.mkdir('.bzr')
-        t.put('.bzr/branch-format', StringIO())
+        t.put_bytes('.bzr/branch-format', '')
         self.assertRaises(UnknownFormatError,
                           bzrdir.BzrDirFormat.find_format,
                           get_transport('.'))

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2006-08-21 19:05:37 +0000
+++ bzrlib/tests/test_errors.py	2006-09-08 01:58:47 +0000
@@ -31,6 +31,19 @@
         error = errors.NoRepositoryPresent(dir)
         self.assertNotEqual(-1, str(error).find((dir.transport.clone('..').base)))
         self.assertEqual(-1, str(error).find((dir.transport.base)))
+        
+    def test_no_such_id(self):
+        error = errors.NoSuchId("atree", "anid")
+        self.assertEqualDiff("The file id anid is not present in the tree "
+            "atree.",
+            str(error))
+
+    def test_no_such_revision_in_tree(self):
+        error = errors.NoSuchRevisionInTree("atree", "anid")
+        self.assertEqualDiff("The revision id anid is not present in the tree "
+            "atree.",
+            str(error))
+        self.assertIsInstance(error, errors.NoSuchRevision)
 
     def test_up_to_date(self):
         error = errors.UpToDateFormat(bzrdir.BzrDirFormat4())

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2006-07-24 20:05:07 +0000
+++ bzrlib/tests/test_knit.py	2006-09-06 20:54:07 +0000
@@ -20,7 +20,7 @@
 import difflib
 
 
-from bzrlib.errors import KnitError, RevisionAlreadyPresent
+from bzrlib.errors import KnitError, RevisionAlreadyPresent, NoSuchFile
 from bzrlib.knit import (
     KnitVersionedFile,
     KnitPlainFactory,
@@ -36,12 +36,14 @@
 class KnitTests(TestCaseWithTransport):
     """Class containing knit test helper routines."""
 
-    def make_test_knit(self, annotate=False):
+    def make_test_knit(self, annotate=False, delay_create=False):
         if not annotate:
             factory = KnitPlainFactory()
         else:
             factory = None
-        return KnitVersionedFile('test', get_transport('.'), access_mode='w', factory=factory, create=True)
+        return KnitVersionedFile('test', get_transport('.'), access_mode='w',
+                                 factory=factory, create=True,
+                                 delay_create=delay_create)
 
 
 class BasicKnitTests(KnitTests):
@@ -312,6 +314,7 @@
         # this tests that a new knit index file has the expected content
         # and that is writes the data we expect as records are added.
         knit = self.make_test_knit(True)
+        # Now knit files are not created until we first add data to them
         self.assertFileEqual("# bzr knit index 8\n", 'test.kndx')
         knit.add_lines_with_ghosts('revid', ['a_ghost'], ['a\n'])
         self.assertFileEqual(
@@ -342,6 +345,91 @@
         self.assertEqual(['revid', 'revid2', 'revid3'], knit.versions())
         self.assertEqual(['revid2'], knit.get_parents('revid3'))
 
+    def test_delay_create(self):
+        """Test that passing delay_create=True creates files late"""
+        knit = self.make_test_knit(annotate=True, delay_create=True)
+        self.failIfExists('test.knit')
+        self.failIfExists('test.kndx')
+        knit.add_lines_with_ghosts('revid', ['a_ghost'], ['a\n'])
+        self.failUnlessExists('test.knit')
+        self.assertFileEqual(
+            "# bzr knit index 8\n"
+            "\n"
+            "revid fulltext 0 84 .a_ghost :",
+            'test.kndx')
+
+    def test_create_parent_dir(self):
+        """create_parent_dir can create knits in nonexistant dirs"""
+        # Has no effect if we don't set 'delay_create'
+        trans = get_transport('.')
+        self.assertRaises(NoSuchFile, KnitVersionedFile, 'dir/test',
+                          trans, access_mode='w', factory=None,
+                          create=True, create_parent_dir=True)
+        # Nothing should have changed yet
+        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+                                 factory=None, create=True,
+                                 create_parent_dir=True,
+                                 delay_create=True)
+        self.failIfExists('dir/test.knit')
+        self.failIfExists('dir/test.kndx')
+        self.failIfExists('dir')
+        knit.add_lines('revid', [], ['a\n'])
+        self.failUnlessExists('dir')
+        self.failUnlessExists('dir/test.knit')
+        self.assertFileEqual(
+            "# bzr knit index 8\n"
+            "\n"
+            "revid fulltext 0 84  :",
+            'dir/test.kndx')
+
+    def test_create_mode_700(self):
+        trans = get_transport('.')
+        if not trans._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+                                 factory=None, create=True,
+                                 create_parent_dir=True,
+                                 delay_create=True,
+                                 file_mode=0600,
+                                 dir_mode=0700)
+        knit.add_lines('revid', [], ['a\n'])
+        self.assertTransportMode(trans, 'dir', 0700)
+        self.assertTransportMode(trans, 'dir/test.knit', 0600)
+        self.assertTransportMode(trans, 'dir/test.kndx', 0600)
+
+    def test_create_mode_770(self):
+        trans = get_transport('.')
+        if not trans._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+                                 factory=None, create=True,
+                                 create_parent_dir=True,
+                                 delay_create=True,
+                                 file_mode=0660,
+                                 dir_mode=0770)
+        knit.add_lines('revid', [], ['a\n'])
+        self.assertTransportMode(trans, 'dir', 0770)
+        self.assertTransportMode(trans, 'dir/test.knit', 0660)
+        self.assertTransportMode(trans, 'dir/test.kndx', 0660)
+
+    def test_create_mode_777(self):
+        trans = get_transport('.')
+        if not trans._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+                                 factory=None, create=True,
+                                 create_parent_dir=True,
+                                 delay_create=True,
+                                 file_mode=0666,
+                                 dir_mode=0777)
+        knit.add_lines('revid', [], ['a\n'])
+        self.assertTransportMode(trans, 'dir', 0777)
+        self.assertTransportMode(trans, 'dir/test.knit', 0666)
+        self.assertTransportMode(trans, 'dir/test.kndx', 0666)
+
     def test_plan_merge(self):
         my_knit = self.make_test_knit(annotate=True)
         my_knit.add_lines('text1', [], split_lines(TEXT_1))

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2006-08-25 05:03:17 +0000
+++ bzrlib/tests/test_merge.py	2006-09-06 02:47:54 +0000
@@ -17,6 +17,7 @@
 import os
 from StringIO import StringIO
 
+from bzrlib import conflicts
 from bzrlib.branch import Branch
 from bzrlib.builtins import merge
 from bzrlib.conflicts import ConflictList, TextConflict
@@ -148,3 +149,27 @@
         tree_a.set_conflicts(ConflictList([TextConflict('patha')]))
         merge_inner(tree_a.branch, tree_a, tree_a, this_tree=tree_a)
         self.assertEqual(1, len(tree_a.conflicts()))
+
+    def test_rmdir_conflict(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/b/'])
+        tree_a.add('b', 'b-id')
+        tree_a.commit('added b')
+        base_tree = tree_a.basis_tree()
+        tree_z = tree_a.bzrdir.sprout('z').open_workingtree()
+        self.build_tree(['a/b/c'])
+        tree_a.add('b/c')
+        tree_a.commit('added c')
+        os.rmdir('z/b')
+        tree_z.commit('removed b')
+        merge_inner(tree_z.branch, tree_a, base_tree, this_tree=tree_z)
+        self.assertEqual([
+            conflicts.MissingParent('Created directory', 'b', 'b-id'),
+            conflicts.UnversionedParent('Versioned directory', 'b', 'b-id')],
+            tree_z.conflicts())
+        merge_inner(tree_a.branch, tree_z.basis_tree(), base_tree, 
+                    this_tree=tree_a)
+        self.assertEqual([
+            conflicts.DeletingParent('Not deleting', 'b', 'b-id'),
+            conflicts.UnversionedParent('Versioned directory', 'b', 'b-id')],
+            tree_a.conflicts())

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-08-14 22:53:25 +0000
+++ bzrlib/tests/test_osutils.py	2006-09-05 13:57:30 +0000
@@ -529,3 +529,63 @@
         # and in the worst case, use bzrlib.user_encoding
         self.assertEqual('user_encoding', osutils.get_terminal_encoding())
 
+
+class TestSetUnsetEnv(TestCase):
+    """Test updating the environment"""
+
+    def setUp(self):
+        super(TestSetUnsetEnv, self).setUp()
+
+        self.assertEqual(None, os.environ.get('BZR_TEST_ENV_VAR'),
+                         'Environment was not cleaned up properly.'
+                         ' Variable BZR_TEST_ENV_VAR should not exist.')
+        def cleanup():
+            if 'BZR_TEST_ENV_VAR' in os.environ:
+                del os.environ['BZR_TEST_ENV_VAR']
+
+        self.addCleanup(cleanup)
+
+    def test_set(self):
+        """Test that we can set an env variable"""
+        old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', 'foo')
+        self.assertEqual(None, old)
+        self.assertEqual('foo', os.environ.get('BZR_TEST_ENV_VAR'))
+
+    def test_double_set(self):
+        """Test that we get the old value out"""
+        osutils.set_or_unset_env('BZR_TEST_ENV_VAR', 'foo')
+        old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', 'bar')
+        self.assertEqual('foo', old)
+        self.assertEqual('bar', os.environ.get('BZR_TEST_ENV_VAR'))
+
+    def test_unicode(self):
+        """Environment can only contain plain strings
+        
+        So Unicode strings must be encoded.
+        """
+        # Try a few different characters, to see if we can get
+        # one that will be valid in the user_encoding
+        possible_vals = [u'm\xb5', u'\xe1', u'\u0410']
+        for uni_val in possible_vals:
+            try:
+                env_val = uni_val.encode(bzrlib.user_encoding)
+            except UnicodeEncodeError:
+                # Try a different character
+                pass
+            else:
+                break
+        else:
+            raise TestSkipped('Cannot find a unicode character that works in'
+                              ' encoding %s' % (bzrlib.user_encoding,))
+
+        old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', uni_val)
+        self.assertEqual(env_val, os.environ.get('BZR_TEST_ENV_VAR'))
+
+    def test_unset(self):
+        """Test that passing None will remove the env var"""
+        osutils.set_or_unset_env('BZR_TEST_ENV_VAR', 'foo')
+        old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', None)
+        self.assertEqual('foo', old)
+        self.assertEqual(None, os.environ.get('BZR_TEST_ENV_VAR'))
+        self.failIf('BZR_TEST_ENV_VAR' in os.environ)
+

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_permissions.py	2006-09-05 20:58:28 +0000
@@ -33,7 +33,7 @@
 import os
 import sys
 import stat
-from StringIO import StringIO
+from cStringIO import StringIO
 
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
@@ -156,7 +156,7 @@
         #                    extra chmod calls aren't being made
         try:
             transport = get_transport(self.get_url())
-            transport.put('my-lock', StringIO(''))
+            transport.put_bytes('my-lock', '')
             lockable = LockableFiles(transport, 'my-lock', TransportLock)
             self.assertNotEqual(None, lockable._dir_mode)
             self.assertNotEqual(None, lockable._file_mode)
@@ -276,7 +276,7 @@
             self.assertTransportMode(t, 'a', 0666 &~umask)
 
             # but Transport overrides umask
-            t.put('b', 'txt', mode=0666)
+            t.put_bytes('b', 'txt', mode=0666)
             self.assertTransportMode(t, 'b', 0666)
 
             t._sftp.mkdir('c', mode=0777)

=== modified file 'bzrlib/tests/test_read_bundle.py'
--- bzrlib/tests/test_read_bundle.py	2006-09-05 08:14:23 +0000
+++ bzrlib/tests/test_read_bundle.py	2006-09-08 00:19:48 +0000
@@ -64,7 +64,7 @@
             f.write(out.getvalue())
             f.close()
         else:
-            self._transport.put('test_bundle', out)
+            self._transport.put_file('test_bundle', out)
             self.log('Put to: %s', self.get_url('test_bundle'))
         return wt
 

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2006-07-10 20:22:10 +0000
+++ bzrlib/tests/test_repository.py	2006-09-05 20:58:28 +0000
@@ -72,7 +72,7 @@
     def initialize(self, a_bzrdir, shared=False):
         """Initialize a repository in a BzrDir"""
         t = a_bzrdir.get_repository_transport(self)
-        t.put('format', StringIO(self.get_format_string()))
+        t.put_bytes('format', self.get_format_string())
         return 'A bzr repository dir'
 
     def is_supported(self):

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-09-05 23:08:03 +0000
+++ bzrlib/tests/test_selftest.py	2006-09-08 00:19:48 +0000
@@ -420,6 +420,16 @@
         self.assertEqual(tests[1].transport_server, server1)
         self.assertEqual(tests[1].transport_readonly_server, server2)
 
+
+class TestTestCaseInTempDir(TestCaseInTempDir):
+
+    def test_home_is_not_working(self):
+        self.assertNotEqual(self.test_dir, self.test_home_dir)
+        cwd = osutils.getcwd()
+        self.assertEqual(self.test_dir, cwd)
+        self.assertEqual(self.test_home_dir, os.environ['HOME'])
+
+
 class TestTestCaseWithTransport(TestCaseWithTransport):
     """Tests for the convenience functions TestCaseWithTransport introduces."""
 

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2006-08-15 15:50:31 +0000
+++ bzrlib/tests/test_smart_add.py	2006-09-07 21:37:43 +0000
@@ -121,7 +121,6 @@
     def test_add_dry_run(self):
         """Test a dry run add, make sure nothing is added."""
         from bzrlib.commands import run_bzr
-        ignores._set_user_ignores(['./.bazaar'])
         eq = self.assertEqual
         wt = self.make_branch_and_tree('.')
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
@@ -140,7 +139,7 @@
         from bzrlib.commands import run_bzr
         wt = self.make_branch_and_tree('.')
         # The default ignore list includes '*.py[co]', but not CVS
-        ignores._set_user_ignores(['./.bazaar', '*.py[co]'])
+        ignores._set_user_ignores(['*.py[co]'])
         self.build_tree(['inertiatic/', 'inertiatic/esp', 'inertiatic/CVS',
                         'inertiatic/foo.pyc'])
         added, ignored = smart_add_tree(wt, u'.')

=== modified file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_store.py	2006-09-05 20:58:28 +0000
@@ -38,6 +38,14 @@
         f = store.get(fileid)
         self.assertEqual(f.read(), value)
 
+    def test_add_str_deprecated(self):
+        os.mkdir('a')
+        store = self.get_store('a')
+        self.callDeprecated(['Passing a string to Store.add'
+                             ' was deprecated in version 0.11.'],
+                            store.add, 'foo', '1')
+        self.assertEqual('foo', store.get('1').read())
+
     def fill_store(self, store):
         store.add(StringIO('hello'), 'a')
         store.add(StringIO('other'), 'b')
@@ -48,7 +56,7 @@
         """Test copying"""
         os.mkdir('a')
         store_a = self.get_store('a')
-        store_a.add('foo', '1')
+        store_a.add(StringIO('foo'), '1')
         os.mkdir('b')
         store_b = self.get_store('b')
         store_b.copy_all_ids(store_a)

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-08-30 03:54:34 +0000
+++ bzrlib/tests/test_transform.py	2006-09-06 12:00:01 +0000
@@ -19,7 +19,7 @@
 from bzrlib import tests
 from bzrlib.bzrdir import BzrDir
 from bzrlib.conflicts import (DuplicateEntry, DuplicateID, MissingParent,
-                              UnversionedParent, ParentLoop)
+                              UnversionedParent, ParentLoop, DeletingParent,)
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
                            ReusingTransform, CantMoveRoot, 
                            PathsNotVersionedError, ExistingLimbo,
@@ -388,9 +388,13 @@
                                          'dorothy-id')
         old_dorothy = conflicts.trans_id_tree_file_id('dorothy-id')
         oz = conflicts.trans_id_tree_file_id('oz-id')
-        # set up missing, unversioned parent
+        # set up DeletedParent parent conflict
         conflicts.delete_versioned(oz)
         emerald = conflicts.trans_id_tree_file_id('emerald-id')
+        # set up MissingParent conflict
+        munchkincity = conflicts.trans_id_file_id('munchkincity-id')
+        conflicts.adjust_path('munchkincity', root, munchkincity)
+        conflicts.new_directory('auntem', munchkincity, 'auntem-id')
         # set up parent loop
         conflicts.adjust_path('emeraldcity', emerald, emerald)
         return conflicts, emerald, oz, old_dorothy, new_dorothy
@@ -417,15 +421,22 @@
                                    'dorothy.moved', 'dorothy', None,
                                    'dorothy-id')
         self.assertEqual(cooked_conflicts[1], duplicate_id)
-        missing_parent = MissingParent('Not deleting', 'oz', 'oz-id')
+        missing_parent = MissingParent('Created directory', 'munchkincity',
+                                       'munchkincity-id')
+        deleted_parent = DeletingParent('Not deleting', 'oz', 'oz-id')
         self.assertEqual(cooked_conflicts[2], missing_parent)
-        unversioned_parent = UnversionedParent('Versioned directory', 'oz',
+        unversioned_parent = UnversionedParent('Versioned directory',
+                                               'munchkincity',
+                                               'munchkincity-id')
+        unversioned_parent2 = UnversionedParent('Versioned directory', 'oz',
                                                'oz-id')
         self.assertEqual(cooked_conflicts[3], unversioned_parent)
         parent_loop = ParentLoop('Cancelled move', 'oz/emeraldcity', 
                                  'oz/emeraldcity', 'emerald-id', 'emerald-id')
-        self.assertEqual(cooked_conflicts[4], parent_loop)
-        self.assertEqual(len(cooked_conflicts), 5)
+        self.assertEqual(cooked_conflicts[4], deleted_parent)
+        self.assertEqual(cooked_conflicts[5], unversioned_parent2)
+        self.assertEqual(cooked_conflicts[6], parent_loop)
+        self.assertEqual(len(cooked_conflicts), 7)
         tt.finalize()
 
     def test_string_conflicts(self):
@@ -441,11 +452,17 @@
         self.assertEqual(conflicts_s[1], 'Conflict adding id to dorothy.  '
                                          'Unversioned existing file '
                                          'dorothy.moved.')
-        self.assertEqual(conflicts_s[2], 'Conflict adding files to oz.  '
-                                         'Not deleting.')
-        self.assertEqual(conflicts_s[3], 'Conflict adding versioned files to '
-                                         'oz.  Versioned directory.')
-        self.assertEqual(conflicts_s[4], 'Conflict moving oz/emeraldcity into'
+        self.assertEqual(conflicts_s[2], 'Conflict adding files to'
+                                         ' munchkincity.  Created directory.')
+        self.assertEqual(conflicts_s[3], 'Conflict because munchkincity is not'
+                                         ' versioned, but has versioned'
+                                         ' children.  Versioned directory.')
+        self.assertEqualDiff(conflicts_s[4], "Conflict: can't delete oz because it"
+                                         " is not empty.  Not deleting.")
+        self.assertEqual(conflicts_s[5], 'Conflict because oz is not'
+                                         ' versioned, but has versioned'
+                                         ' children.  Versioned directory.')
+        self.assertEqual(conflicts_s[6], 'Conflict moving oz/emeraldcity into'
                                          ' oz/emeraldcity.  Cancelled move.')
 
     def test_moving_versioned_directories(self):

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2006-07-28 16:52:19 +0000
+++ bzrlib/tests/test_transport.py	2006-09-05 21:25:15 +0000
@@ -178,27 +178,27 @@
 
     def test_append_and_get(self):
         transport = MemoryTransport()
-        transport.append('path', StringIO('content'))
+        transport.append_bytes('path', 'content')
         self.assertEqual(transport.get('path').read(), 'content')
-        transport.append('path', StringIO('content'))
+        transport.append_file('path', StringIO('content'))
         self.assertEqual(transport.get('path').read(), 'contentcontent')
 
     def test_put_and_get(self):
         transport = MemoryTransport()
-        transport.put('path', StringIO('content'))
+        transport.put_file('path', StringIO('content'))
         self.assertEqual(transport.get('path').read(), 'content')
-        transport.put('path', StringIO('content'))
+        transport.put_bytes('path', 'content')
         self.assertEqual(transport.get('path').read(), 'content')
 
     def test_append_without_dir_fails(self):
         transport = MemoryTransport()
         self.assertRaises(NoSuchFile,
-                          transport.append, 'dir/path', StringIO('content'))
+                          transport.append_bytes, 'dir/path', 'content')
 
     def test_put_without_dir_fails(self):
         transport = MemoryTransport()
         self.assertRaises(NoSuchFile,
-                          transport.put, 'dir/path', StringIO('content'))
+                          transport.put_file, 'dir/path', StringIO('content'))
 
     def test_get_missing(self):
         transport = MemoryTransport()
@@ -210,13 +210,13 @@
 
     def test_has_present(self):
         transport = MemoryTransport()
-        transport.append('foo', StringIO('content'))
+        transport.append_bytes('foo', 'content')
         self.assertEquals(True, transport.has('foo'))
 
     def test_mkdir(self):
         transport = MemoryTransport()
         transport.mkdir('dir')
-        transport.append('dir/path', StringIO('content'))
+        transport.append_bytes('dir/path', 'content')
         self.assertEqual(transport.get('dir/path').read(), 'content')
 
     def test_mkdir_missing_parent(self):
@@ -238,16 +238,16 @@
     def test_iter_files_recursive(self):
         transport = MemoryTransport()
         transport.mkdir('dir')
-        transport.put('dir/foo', StringIO('content'))
-        transport.put('dir/bar', StringIO('content'))
-        transport.put('bar', StringIO('content'))
+        transport.put_bytes('dir/foo', 'content')
+        transport.put_bytes('dir/bar', 'content')
+        transport.put_bytes('bar', 'content')
         paths = set(transport.iter_files_recursive())
         self.assertEqual(set(['dir/foo', 'dir/bar', 'bar']), paths)
 
     def test_stat(self):
         transport = MemoryTransport()
-        transport.put('foo', StringIO('content'))
-        transport.put('bar', StringIO('phowar'))
+        transport.put_bytes('foo', 'content')
+        transport.put_bytes('bar', 'phowar')
         self.assertEqual(7, transport.stat('foo').st_size)
         self.assertEqual(6, transport.stat('bar').st_size)
 

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-09-01 17:20:48 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-09-06 20:42:45 +0000
@@ -34,6 +34,7 @@
                            TransportNotPossible, ConnectionError,
                            InvalidURL)
 from bzrlib.osutils import getcwd
+from bzrlib.symbol_versioning import zero_eleven
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
 from bzrlib.tests.test_transport import TestTransportImplementation
 from bzrlib.transport import memory
@@ -111,64 +112,292 @@
         self.assertListRaises(NoSuchFile, t.get_multi, ['a', 'b', 'c'])
         self.assertListRaises(NoSuchFile, t.get_multi, iter(['a', 'b', 'c']))
 
+    def test_get_bytes(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g']
+        contents = ['contents of a\n',
+                    'contents of b\n',
+                    'contents of e\n',
+                    'contents of g\n',
+                    ]
+        self.build_tree(files, transport=t, line_endings='binary')
+        self.check_transport_contents('contents of a\n', t, 'a')
+
+        for content, fname in zip(contents, files):
+            self.assertEqual(content, t.get_bytes(fname))
+
+        self.assertRaises(NoSuchFile, t.get_bytes, 'c')
+
     def test_put(self):
         t = self.get_transport()
 
         if t.is_readonly():
-            self.assertRaises(TransportNotPossible,
-                    t.put, 'a', 'some text for a\n')
-            return
-
-        t.put('a', StringIO('some text for a\n'))
-        self.failUnless(t.has('a'))
-        self.check_transport_contents('some text for a\n', t, 'a')
-        # Make sure 'has' is updated
-        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
-                [True, False, False, False, False])
-        # Put also replaces contents
-        self.assertEqual(t.put_multi([('a', StringIO('new\ncontents for\na\n')),
-                                      ('d', StringIO('contents\nfor d\n'))]),
-                         2)
-        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
-                [True, False, False, True, False])
+            return
+
+        self.applyDeprecated(zero_eleven, t.put, 'a', 'string\ncontents\n')
+        self.check_transport_contents('string\ncontents\n', t, 'a')
+
+        self.applyDeprecated(zero_eleven,
+                             t.put, 'b', StringIO('file-like\ncontents\n'))
+        self.check_transport_contents('file-like\ncontents\n', t, 'b')
+
+    def test_put_bytes(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.put_bytes, 'a', 'some text for a\n')
+            return
+
+        t.put_bytes('a', 'some text for a\n')
+        self.failUnless(t.has('a'))
+        self.check_transport_contents('some text for a\n', t, 'a')
+
+        # The contents should be overwritten
+        t.put_bytes('a', 'new text for a\n')
+        self.check_transport_contents('new text for a\n', t, 'a')
+
+        self.assertRaises(NoSuchFile,
+                          t.put_bytes, 'path/doesnt/exist/c', 'contents')
+
+    def test_put_bytes_non_atomic(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.put_bytes_non_atomic, 'a', 'some text for a\n')
+            return
+
+        self.failIf(t.has('a'))
+        t.put_bytes_non_atomic('a', 'some text for a\n')
+        self.failUnless(t.has('a'))
+        self.check_transport_contents('some text for a\n', t, 'a')
+        # Put also replaces contents
+        t.put_bytes_non_atomic('a', 'new\ncontents for\na\n')
+        self.check_transport_contents('new\ncontents for\na\n', t, 'a')
+
+        # Make sure we can create another file
+        t.put_bytes_non_atomic('d', 'contents for\nd\n')
+        # And overwrite 'a' with empty contents
+        t.put_bytes_non_atomic('a', '')
+        self.check_transport_contents('contents for\nd\n', t, 'd')
+        self.check_transport_contents('', t, 'a')
+
+        self.assertRaises(NoSuchFile, t.put_bytes_non_atomic, 'no/such/path',
+                                       'contents\n')
+        # Now test the create_parent flag
+        self.assertRaises(NoSuchFile, t.put_bytes_non_atomic, 'dir/a',
+                                       'contents\n')
+        self.failIf(t.has('dir/a'))
+        t.put_bytes_non_atomic('dir/a', 'contents for dir/a\n',
+                               create_parent_dir=True)
+        self.check_transport_contents('contents for dir/a\n', t, 'dir/a')
+        
+        # But we still get NoSuchFile if we can't make the parent dir
+        self.assertRaises(NoSuchFile, t.put_bytes_non_atomic, 'not/there/a',
+                                       'contents\n',
+                                       create_parent_dir=True)
+
+    def test_put_bytes_permissions(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        if not t._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        t.put_bytes('mode644', 'test text\n', mode=0644)
+        self.assertTransportMode(t, 'mode644', 0644)
+        t.put_bytes('mode666', 'test text\n', mode=0666)
+        self.assertTransportMode(t, 'mode666', 0666)
+        t.put_bytes('mode600', 'test text\n', mode=0600)
+        self.assertTransportMode(t, 'mode600', 0600)
+        # Yes, you can put_bytes a file such that it becomes readonly
+        t.put_bytes('mode400', 'test text\n', mode=0400)
+        self.assertTransportMode(t, 'mode400', 0400)
+
+        # The default permissions should be based on the current umask
+        umask = osutils.get_umask()
+        t.put_bytes('nomode', 'test text\n', mode=None)
+        self.assertTransportMode(t, 'nomode', 0666 & ~umask)
+        
+    def test_put_bytes_non_atomic_permissions(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        if not t._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        t.put_bytes_non_atomic('mode644', 'test text\n', mode=0644)
+        self.assertTransportMode(t, 'mode644', 0644)
+        t.put_bytes_non_atomic('mode666', 'test text\n', mode=0666)
+        self.assertTransportMode(t, 'mode666', 0666)
+        t.put_bytes_non_atomic('mode600', 'test text\n', mode=0600)
+        self.assertTransportMode(t, 'mode600', 0600)
+        t.put_bytes_non_atomic('mode400', 'test text\n', mode=0400)
+        self.assertTransportMode(t, 'mode400', 0400)
+
+        # The default permissions should be based on the current umask
+        umask = osutils.get_umask()
+        t.put_bytes_non_atomic('nomode', 'test text\n', mode=None)
+        self.assertTransportMode(t, 'nomode', 0666 & ~umask)
+
+        # We should also be able to set the mode for a parent directory
+        # when it is created
+        t.put_bytes_non_atomic('dir700/mode664', 'test text\n', mode=0664,
+                               dir_mode=0700, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir700', 0700)
+        t.put_bytes_non_atomic('dir770/mode664', 'test text\n', mode=0664,
+                               dir_mode=0770, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir770', 0770)
+        t.put_bytes_non_atomic('dir777/mode664', 'test text\n', mode=0664,
+                               dir_mode=0777, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir777', 0777)
+        
+    def test_put_file(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.put_file, 'a', StringIO('some text for a\n'))
+            return
+
+        t.put_file('a', StringIO('some text for a\n'))
+        self.failUnless(t.has('a'))
+        self.check_transport_contents('some text for a\n', t, 'a')
+        # Put also replaces contents
+        t.put_file('a', StringIO('new\ncontents for\na\n'))
+        self.check_transport_contents('new\ncontents for\na\n', t, 'a')
+        self.assertRaises(NoSuchFile,
+                          t.put_file, 'path/doesnt/exist/c',
+                              StringIO('contents'))
+
+    def test_put_file_non_atomic(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.put_file_non_atomic, 'a', StringIO('some text for a\n'))
+            return
+
+        self.failIf(t.has('a'))
+        t.put_file_non_atomic('a', StringIO('some text for a\n'))
+        self.failUnless(t.has('a'))
+        self.check_transport_contents('some text for a\n', t, 'a')
+        # Put also replaces contents
+        t.put_file_non_atomic('a', StringIO('new\ncontents for\na\n'))
+        self.check_transport_contents('new\ncontents for\na\n', t, 'a')
+
+        # Make sure we can create another file
+        t.put_file_non_atomic('d', StringIO('contents for\nd\n'))
+        # And overwrite 'a' with empty contents
+        t.put_file_non_atomic('a', StringIO(''))
+        self.check_transport_contents('contents for\nd\n', t, 'd')
+        self.check_transport_contents('', t, 'a')
+
+        self.assertRaises(NoSuchFile, t.put_file_non_atomic, 'no/such/path',
+                                       StringIO('contents\n'))
+        # Now test the create_parent flag
+        self.assertRaises(NoSuchFile, t.put_file_non_atomic, 'dir/a',
+                                       StringIO('contents\n'))
+        self.failIf(t.has('dir/a'))
+        t.put_file_non_atomic('dir/a', StringIO('contents for dir/a\n'),
+                              create_parent_dir=True)
+        self.check_transport_contents('contents for dir/a\n', t, 'dir/a')
+        
+        # But we still get NoSuchFile if we can't make the parent dir
+        self.assertRaises(NoSuchFile, t.put_file_non_atomic, 'not/there/a',
+                                       StringIO('contents\n'),
+                                       create_parent_dir=True)
+
+    def test_put_file_permissions(self):
+
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        if not t._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        t.put_file('mode644', StringIO('test text\n'), mode=0644)
+        self.assertTransportMode(t, 'mode644', 0644)
+        t.put_file('mode666', StringIO('test text\n'), mode=0666)
+        self.assertTransportMode(t, 'mode666', 0666)
+        t.put_file('mode600', StringIO('test text\n'), mode=0600)
+        self.assertTransportMode(t, 'mode600', 0600)
+        # Yes, you can put a file such that it becomes readonly
+        t.put_file('mode400', StringIO('test text\n'), mode=0400)
+        self.assertTransportMode(t, 'mode400', 0400)
+
+        # XXX: put_multi is deprecated, so do we really care anymore?
+        self.applyDeprecated(zero_eleven, t.put_multi,
+                             [('mmode644', StringIO('text\n'))], mode=0644)
+        self.assertTransportMode(t, 'mmode644', 0644)
+
+        # The default permissions should be based on the current umask
+        umask = osutils.get_umask()
+        t.put_file('nomode', StringIO('test text\n'), mode=None)
+        self.assertTransportMode(t, 'nomode', 0666 & ~umask)
+        
+    def test_put_file_non_atomic_permissions(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        if not t._can_roundtrip_unix_modebits():
+            # Can't roundtrip, so no need to run this test
+            return
+        t.put_file_non_atomic('mode644', StringIO('test text\n'), mode=0644)
+        self.assertTransportMode(t, 'mode644', 0644)
+        t.put_file_non_atomic('mode666', StringIO('test text\n'), mode=0666)
+        self.assertTransportMode(t, 'mode666', 0666)
+        t.put_file_non_atomic('mode600', StringIO('test text\n'), mode=0600)
+        self.assertTransportMode(t, 'mode600', 0600)
+        # Yes, you can put_file_non_atomic a file such that it becomes readonly
+        t.put_file_non_atomic('mode400', StringIO('test text\n'), mode=0400)
+        self.assertTransportMode(t, 'mode400', 0400)
+
+        # The default permissions should be based on the current umask
+        umask = osutils.get_umask()
+        t.put_file_non_atomic('nomode', StringIO('test text\n'), mode=None)
+        self.assertTransportMode(t, 'nomode', 0666 & ~umask)
+        
+        # We should also be able to set the mode for a parent directory
+        # when it is created
+        sio = StringIO()
+        t.put_file_non_atomic('dir700/mode664', sio, mode=0664,
+                              dir_mode=0700, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir700', 0700)
+        t.put_file_non_atomic('dir770/mode664', sio, mode=0664,
+                              dir_mode=0770, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir770', 0770)
+        t.put_file_non_atomic('dir777/mode664', sio, mode=0664,
+                              dir_mode=0777, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir777', 0777)
+
+    def test_put_multi(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        self.assertEqual(2, self.applyDeprecated(zero_eleven,
+            t.put_multi, [('a', StringIO('new\ncontents for\na\n')),
+                          ('d', StringIO('contents\nfor d\n'))]
+            ))
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd'])),
+                [True, False, False, True])
         self.check_transport_contents('new\ncontents for\na\n', t, 'a')
         self.check_transport_contents('contents\nfor d\n', t, 'd')
 
-        self.assertEqual(
-            t.put_multi(iter([('a', StringIO('diff\ncontents for\na\n')),
-                              ('d', StringIO('another contents\nfor d\n'))])),
-                        2)
+        self.assertEqual(2, self.applyDeprecated(zero_eleven,
+            t.put_multi, iter([('a', StringIO('diff\ncontents for\na\n')),
+                              ('d', StringIO('another contents\nfor d\n'))])
+            ))
         self.check_transport_contents('diff\ncontents for\na\n', t, 'a')
         self.check_transport_contents('another contents\nfor d\n', t, 'd')
 
-        self.assertRaises(NoSuchFile,
-                          t.put, 'path/doesnt/exist/c', StringIO('contents'))
-
-    def test_put_permissions(self):
-        t = self.get_transport()
-
-        if t.is_readonly():
-            return
-        if not t._can_roundtrip_unix_modebits():
-            # Can't roundtrip, so no need to run this test
-            return
-        t.put('mode644', StringIO('test text\n'), mode=0644)
-        self.assertTransportMode(t, 'mode644', 0644)
-        t.put('mode666', StringIO('test text\n'), mode=0666)
-        self.assertTransportMode(t, 'mode666', 0666)
-        t.put('mode600', StringIO('test text\n'), mode=0600)
-        self.assertTransportMode(t, 'mode600', 0600)
-        # Yes, you can put a file such that it becomes readonly
-        t.put('mode400', StringIO('test text\n'), mode=0400)
-        self.assertTransportMode(t, 'mode400', 0400)
-        t.put_multi([('mmode644', StringIO('text\n'))], mode=0644)
-        self.assertTransportMode(t, 'mmode644', 0644)
-
-        # The default permissions should be based on the current umask
-        umask = osutils.get_umask()
-        t.put('nomode', StringIO('test text\n'), mode=None)
-        self.assertTransportMode(t, 'nomode', 0666 & ~umask)
-        
     def test_mkdir(self):
         t = self.get_transport()
 
@@ -207,9 +436,8 @@
         self.assertRaises(FileExists, t.mkdir, 'dir_g')
 
         # Test get/put in sub-directories
-        self.assertEqual(2, 
-            t.put_multi([('dir_a/a', StringIO('contents of dir_a/a')),
-                         ('dir_b/b', StringIO('contents of dir_b/b'))]))
+        t.put_bytes('dir_a/a', 'contents of dir_a/a')
+        t.put_file('dir_b/b', StringIO('contents of dir_b/b'))
         self.check_transport_contents('contents of dir_a/a', t, 'dir_a/a')
         self.check_transport_contents('contents of dir_b/b', t, 'dir_b/b')
 
@@ -270,7 +498,7 @@
             self.build_tree(['e/', 'e/f'])
         else:
             t.mkdir('e')
-            t.put('e/f', StringIO('contents of e'))
+            t.put_bytes('e/f', 'contents of e')
         self.assertRaises(NoSuchFile, t.copy_to, ['e/f'], temp_transport)
         temp_transport.mkdir('e')
         t.copy_to(['e/f'], temp_transport)
@@ -295,37 +523,84 @@
         t = self.get_transport()
 
         if t.is_readonly():
-            open('a', 'wb').write('diff\ncontents for\na\n')
-            open('b', 'wb').write('contents\nfor b\n')
-        else:
-            t.put_multi([
-                    ('a', StringIO('diff\ncontents for\na\n')),
-                    ('b', StringIO('contents\nfor b\n'))
-                    ])
-
-        if t.is_readonly():
-            self.assertRaises(TransportNotPossible,
-                    t.append, 'a', 'add\nsome\nmore\ncontents\n')
-            _append('a', StringIO('add\nsome\nmore\ncontents\n'))
-        else:
-            self.assertEqual(20,
-                t.append('a', StringIO('add\nsome\nmore\ncontents\n')))
-
-        self.check_transport_contents(
-            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n',
-            t, 'a')
-
-        if t.is_readonly():
-            self.assertRaises(TransportNotPossible,
-                    t.append_multi,
-                        [('a', 'and\nthen\nsome\nmore\n'),
-                         ('b', 'some\nmore\nfor\nb\n')])
-            _append('a', StringIO('and\nthen\nsome\nmore\n'))
-            _append('b', StringIO('some\nmore\nfor\nb\n'))
-        else:
-            self.assertEqual((43, 15), 
-                t.append_multi([('a', StringIO('and\nthen\nsome\nmore\n')),
-                                ('b', StringIO('some\nmore\nfor\nb\n'))]))
+            return
+        t.put_bytes('a', 'diff\ncontents for\na\n')
+        t.put_bytes('b', 'contents\nfor b\n')
+
+        self.assertEqual(20, self.applyDeprecated(zero_eleven,
+            t.append, 'a', StringIO('add\nsome\nmore\ncontents\n')))
+
+        self.check_transport_contents(
+            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n',
+            t, 'a')
+
+        # And we can create new files, too
+        self.assertEqual(0, self.applyDeprecated(zero_eleven,
+            t.append, 'c', StringIO('some text\nfor a missing file\n')))
+        self.check_transport_contents('some text\nfor a missing file\n',
+                                      t, 'c')
+    def test_append_file(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.append_file, 'a', 'add\nsome\nmore\ncontents\n')
+            return
+        t.put_bytes('a', 'diff\ncontents for\na\n')
+        t.put_bytes('b', 'contents\nfor b\n')
+
+        self.assertEqual(20,
+            t.append_file('a', StringIO('add\nsome\nmore\ncontents\n')))
+
+        self.check_transport_contents(
+            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n',
+            t, 'a')
+
+        # a file with no parent should fail..
+        self.assertRaises(NoSuchFile,
+                          t.append_file, 'missing/path', StringIO('content'))
+
+        # And we can create new files, too
+        self.assertEqual(0,
+            t.append_file('c', StringIO('some text\nfor a missing file\n')))
+        self.check_transport_contents('some text\nfor a missing file\n',
+                                      t, 'c')
+
+    def test_append_bytes(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.append_bytes, 'a', 'add\nsome\nmore\ncontents\n')
+            return
+
+        self.assertEqual(0, t.append_bytes('a', 'diff\ncontents for\na\n'))
+        self.assertEqual(0, t.append_bytes('b', 'contents\nfor b\n'))
+
+        self.assertEqual(20,
+            t.append_bytes('a', 'add\nsome\nmore\ncontents\n'))
+
+        self.check_transport_contents(
+            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n',
+            t, 'a')
+
+        # a file with no parent should fail..
+        self.assertRaises(NoSuchFile,
+                          t.append_bytes, 'missing/path', 'content')
+
+    def test_append_multi(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        t.put_bytes('a', 'diff\ncontents for\na\n'
+                         'add\nsome\nmore\ncontents\n')
+        t.put_bytes('b', 'contents\nfor b\n')
+
+        self.assertEqual((43, 15),
+            t.append_multi([('a', StringIO('and\nthen\nsome\nmore\n')),
+                            ('b', StringIO('some\nmore\nfor\nb\n'))]))
+
         self.check_transport_contents(
             'diff\ncontents for\na\n'
             'add\nsome\nmore\ncontents\n'
@@ -336,13 +611,9 @@
                 'some\nmore\nfor\nb\n',
                 t, 'b')
 
-        if t.is_readonly():
-            _append('a', StringIO('a little bit more\n'))
-            _append('b', StringIO('from an iterator\n'))
-        else:
-            self.assertEqual((62, 31),
-                t.append_multi(iter([('a', StringIO('a little bit more\n')),
-                                     ('b', StringIO('from an iterator\n'))])))
+        self.assertEqual((62, 31),
+            t.append_multi(iter([('a', StringIO('a little bit more\n')),
+                                 ('b', StringIO('from an iterator\n'))])))
         self.check_transport_contents(
             'diff\ncontents for\na\n'
             'add\nsome\nmore\ncontents\n'
@@ -355,16 +626,10 @@
                 'from an iterator\n',
                 t, 'b')
 
-        if t.is_readonly():
-            _append('c', StringIO('some text\nfor a missing file\n'))
-            _append('a', StringIO('some text in a\n'))
-            _append('d', StringIO('missing file r\n'))
-        else:
-            self.assertEqual(0,
-                t.append('c', StringIO('some text\nfor a missing file\n')))
-            self.assertEqual((80, 0),
-                t.append_multi([('a', StringIO('some text in a\n')),
-                                ('d', StringIO('missing file r\n'))]))
+        self.assertEqual((80, 0),
+            t.append_multi([('a', StringIO('some text in a\n')),
+                            ('d', StringIO('missing file r\n'))]))
+
         self.check_transport_contents(
             'diff\ncontents for\na\n'
             'add\nsome\nmore\ncontents\n'
@@ -372,123 +637,26 @@
             'a little bit more\n'
             'some text in a\n',
             t, 'a')
-        self.check_transport_contents('some text\nfor a missing file\n',
-                                      t, 'c')
         self.check_transport_contents('missing file r\n', t, 'd')
-        
-        # a file with no parent should fail..
-        if not t.is_readonly():
-            self.assertRaises(NoSuchFile,
-                              t.append, 'missing/path', 
-                              StringIO('content'))
-
-    def test_append_file(self):
-        t = self.get_transport()
-
-        contents = [
-            ('f1', StringIO('this is a string\nand some more stuff\n')),
-            ('f2', StringIO('here is some text\nand a bit more\n')),
-            ('f3', StringIO('some text for the\nthird file created\n')),
-            ('f4', StringIO('this is a string\nand some more stuff\n')),
-            ('f5', StringIO('here is some text\nand a bit more\n')),
-            ('f6', StringIO('some text for the\nthird file created\n'))
-        ]
-        
-        if t.is_readonly():
-            for f, val in contents:
-                open(f, 'wb').write(val.read())
-        else:
-            t.put_multi(contents)
-
-        a1 = StringIO('appending to\none\n')
-        if t.is_readonly():
-            _append('f1', a1)
-        else:
-            t.append('f1', a1)
-
-        del a1
-
-        self.check_transport_contents(
-                'this is a string\nand some more stuff\n'
-                'appending to\none\n',
-                t, 'f1')
-
-        a2 = StringIO('adding more\ntext to two\n')
-        a3 = StringIO('some garbage\nto put in three\n')
-
-        if t.is_readonly():
-            _append('f2', a2)
-            _append('f3', a3)
-        else:
-            t.append_multi([('f2', a2), ('f3', a3)])
-
-        del a2, a3
-
-        self.check_transport_contents(
-                'here is some text\nand a bit more\n'
-                'adding more\ntext to two\n',
-                t, 'f2')
-        self.check_transport_contents( 
-                'some text for the\nthird file created\n'
-                'some garbage\nto put in three\n',
-                t, 'f3')
-
-        # Test that an actual file object can be used with put
-        a4 = t.get('f1')
-        if t.is_readonly():
-            _append('f4', a4)
-        else:
-            t.append('f4', a4)
-
-        del a4
-
-        self.check_transport_contents(
-                'this is a string\nand some more stuff\n'
-                'this is a string\nand some more stuff\n'
-                'appending to\none\n',
-                t, 'f4')
-
-        a5 = t.get('f2')
-        a6 = t.get('f3')
-        if t.is_readonly():
-            _append('f5', a5)
-            _append('f6', a6)
-        else:
-            t.append_multi([('f5', a5), ('f6', a6)])
-
-        del a5, a6
-
-        self.check_transport_contents(
-                'here is some text\nand a bit more\n'
-                'here is some text\nand a bit more\n'
-                'adding more\ntext to two\n',
-                t, 'f5')
-        self.check_transport_contents(
-                'some text for the\nthird file created\n'
-                'some text for the\nthird file created\n'
-                'some garbage\nto put in three\n',
-                t, 'f6')
-
-        a5 = t.get('f2')
-        a6 = t.get('f2')
-        a7 = t.get('f3')
-        if t.is_readonly():
-            _append('c', a5)
-            _append('a', a6)
-            _append('d', a7)
-        else:
-            t.append('c', a5)
-            t.append_multi([('a', a6), ('d', a7)])
-        del a5, a6, a7
-        self.check_transport_contents(t.get('f2').read(), t, 'c')
-        self.check_transport_contents(t.get('f3').read(), t, 'd')
-
-    def test_append_mode(self):
+
+    def test_append_file_mode(self):
+        """Check that append accepts a mode parameter"""
         # check append accepts a mode
         t = self.get_transport()
         if t.is_readonly():
-            return
-        t.append('f', StringIO('f'), mode=None)
+            self.assertRaises(TransportNotPossible,
+                t.append_file, 'f', StringIO('f'), mode=None)
+            return
+        t.append_file('f', StringIO('f'), mode=None)
+        
+    def test_append_bytes_mode(self):
+        # check append_bytes accepts a mode
+        t = self.get_transport()
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                t.append_bytes, 'f', 'f', mode=None)
+            return
+        t.append_bytes('f', 'f', mode=None)
         
     def test_delete(self):
         # TODO: Test Transport.delete
@@ -499,16 +667,16 @@
             self.assertRaises(TransportNotPossible, t.delete, 'missing')
             return
 
-        t.put('a', StringIO('a little bit of text\n'))
+        t.put_bytes('a', 'a little bit of text\n')
         self.failUnless(t.has('a'))
         t.delete('a')
         self.failIf(t.has('a'))
 
         self.assertRaises(NoSuchFile, t.delete, 'a')
 
-        t.put('a', StringIO('a text\n'))
-        t.put('b', StringIO('b text\n'))
-        t.put('c', StringIO('c text\n'))
+        t.put_bytes('a', 'a text\n')
+        t.put_bytes('b', 'b text\n')
+        t.put_bytes('c', 'c text\n')
         self.assertEqual([True, True, True],
                 list(t.has_multi(['a', 'b', 'c'])))
         t.delete_multi(['a', 'c'])
@@ -524,8 +692,8 @@
         self.assertRaises(NoSuchFile,
                 t.delete_multi, iter(['a', 'b', 'c']))
 
-        t.put('a', StringIO('another a text\n'))
-        t.put('c', StringIO('another c text\n'))
+        t.put_bytes('a', 'another a text\n')
+        t.put_bytes('c', 'another c text\n')
         t.delete_multi(iter(['a', 'b', 'c']))
 
         # We should have deleted everything
@@ -629,7 +797,7 @@
         # creates control files in the working directory
         # perhaps all of this could be done in a subdirectory
 
-        t.put('a', StringIO('a first file\n'))
+        t.put_bytes('a', 'a first file\n')
         self.assertEquals([True, False], list(t.has_multi(['a', 'b'])))
 
         t.move('a', 'b')
@@ -640,7 +808,7 @@
         self.assertEquals([False, True], list(t.has_multi(['a', 'b'])))
 
         # Overwrite a file
-        t.put('c', StringIO('c this file\n'))
+        t.put_bytes('c', 'c this file\n')
         t.move('c', 'b')
         self.failIf(t.has('c'))
         self.check_transport_contents('c this file\n', t, 'b')
@@ -655,7 +823,7 @@
         if t.is_readonly():
             return
 
-        t.put('a', StringIO('a file\n'))
+        t.put_bytes('a', 'a file\n')
         t.copy('a', 'b')
         self.check_transport_contents('a file\n', t, 'b')
 
@@ -664,7 +832,7 @@
         # What should the assert be if you try to copy a
         # file over a directory?
         #self.assertRaises(Something, t.copy, 'a', 'c')
-        t.put('d', StringIO('text in d\n'))
+        t.put_bytes('d', 'text in d\n')
         t.copy('d', 'b')
         self.check_transport_contents('text in d\n', t, 'b')
 
@@ -816,7 +984,7 @@
         if t1.is_readonly():
             open('b/d', 'wb').write('newfile\n')
         else:
-            t2.put('d', StringIO('newfile\n'))
+            t2.put_bytes('d', 'newfile\n')
 
         self.failUnless(t1.has('b/d'))
         self.failUnless(t2.has('d'))
@@ -916,8 +1084,6 @@
                               transport.iter_files_recursive)
             return
         if transport.is_readonly():
-            self.assertRaises(TransportNotPossible,
-                              transport.put, 'a', 'some text for a\n')
             return
         self.build_tree(['from/',
                          'from/dir/',
@@ -974,7 +1140,7 @@
         transport = self.get_transport()
         if transport.is_readonly():
             return
-        transport.put('foo', StringIO('bar'))
+        transport.put_bytes('foo', 'bar')
         transport2 = self.get_transport()
         self.check_transport_contents('bar', transport2, 'foo')
         # its base should be usable.
@@ -992,7 +1158,7 @@
         if transport.is_readonly():
             self.assertRaises(TransportNotPossible, transport.lock_write, 'foo')
             return
-        transport.put('lock', StringIO())
+        transport.put_bytes('lock', '')
         lock = transport.lock_write('lock')
         # TODO make this consistent on all platforms:
         # self.assertRaises(LockError, transport.lock_write, 'lock')
@@ -1003,7 +1169,7 @@
         if transport.is_readonly():
             file('lock', 'w').close()
         else:
-            transport.put('lock', StringIO())
+            transport.put_bytes('lock', '')
         lock = transport.lock_read('lock')
         # TODO make this consistent on all platforms:
         # self.assertRaises(LockError, transport.lock_read, 'lock')
@@ -1014,7 +1180,7 @@
         if transport.is_readonly():
             file('a', 'w').write('0123456789')
         else:
-            transport.put('a', StringIO('0123456789'))
+            transport.put_bytes('a', '0123456789')
 
         d = list(transport.readv('a', ((0, 1), (1, 1), (3, 2), (9, 1))))
         self.assertEqual(d[0], (0, '0'))
@@ -1027,7 +1193,7 @@
         if transport.is_readonly():
             file('a', 'w').write('0123456789')
         else:
-            transport.put('a', StringIO('01234567890'))
+            transport.put_bytes('a', '01234567890')
 
         d = list(transport.readv('a', ((1, 1), (9, 1), (0, 1), (3, 2))))
         self.assertEqual(d[0], (1, '1'))

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-08-25 05:03:17 +0000
+++ bzrlib/tests/test_workingtree.py	2006-09-05 20:58:28 +0000
@@ -90,7 +90,7 @@
     def initialize(self, a_bzrdir, revision_id=None):
         """Sample branches cannot be created."""
         t = a_bzrdir.get_workingtree_transport(self)
-        t.put('format', StringIO(self.get_format_string()))
+        t.put_bytes('format', self.get_format_string())
         return 'A tree'
 
     def is_supported(self):

=== modified file 'bzrlib/tests/tree_implementations/__init__.py'
--- bzrlib/tests/tree_implementations/__init__.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/tree_implementations/__init__.py	2006-09-08 01:58:47 +0000
@@ -91,7 +91,8 @@
     def _make_abc_tree(self, tree):
         """setup an abc content tree."""
         files = ['a', 'b/', 'b/c']
-        self.build_tree(files, transport=tree.bzrdir.root_transport)
+        self.build_tree(files, line_endings='binary',
+                        transport=tree.bzrdir.root_transport)
         tree.add(files, ['a-id', 'b-id', 'c-id'])
 
     def get_tree_no_parents_abc_content(self, tree, converter=None):
@@ -189,6 +190,7 @@
     result = TestSuite()
     test_tree_implementations = [
         'bzrlib.tests.tree_implementations.test_test_trees',
+        'bzrlib.tests.tree_implementations.test_revision_tree',
         ]
     adapter = TreeTestProviderAdapter(
         default_transport,

=== modified file 'bzrlib/tests/workingtree_implementations/__init__.py'
--- bzrlib/tests/workingtree_implementations/__init__.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/__init__.py	2006-09-07 07:31:51 +0000
@@ -63,6 +63,7 @@
         'bzrlib.tests.workingtree_implementations.test_merge_from_branch',
         'bzrlib.tests.workingtree_implementations.test_parents',
         'bzrlib.tests.workingtree_implementations.test_pull',
+        'bzrlib.tests.workingtree_implementations.test_unversion',
         'bzrlib.tests.workingtree_implementations.test_workingtree',
         ]
     adapter = WorkingTreeTestProviderAdapter(

=== modified file 'bzrlib/tests/workingtree_implementations/test_commit.py'
--- bzrlib/tests/workingtree_implementations/test_commit.py	2006-09-05 08:14:23 +0000
+++ bzrlib/tests/workingtree_implementations/test_commit.py	2006-09-08 00:19:48 +0000
@@ -114,7 +114,7 @@
         except errors.UpgradeRequired:
             # older format.
             return
-        master.bzrdir.transport.put('branch-format', StringIO('garbage'))
+        master.bzrdir.transport.put_bytes('branch-format', 'garbage')
         del master
         # check its corrupted.
         self.assertRaises(errors.UnknownFormatError,
@@ -163,6 +163,43 @@
             'wibble@fofof--20050401--1928390812'],
             rev.parent_ids)
 
+    def test_commit_deleted_subtree_and_files_updates_workingtree(self):
+        """The working trees inventory may be adjusted by commit."""
+        wt = self.make_branch_and_tree('.')
+        wt.lock_write()
+        self.build_tree(['a', 'b/', 'b/c', 'd'])
+        wt.add(['a', 'b', 'b/c', 'd'], ['a-id', 'b-id', 'c-id', 'd-id'])
+        this_dir = self.get_transport()
+        this_dir.delete_tree('b')
+        this_dir.delete('d')
+        # now we have a tree with a through d in the inventory, but only
+        # a present on disk. After commit b-id, c-id and d-id should be
+        # missing from the inventory, within the same tree transaction.
+        wt.commit('commit stuff')
+        self.assertTrue(wt.has_id('a-id'))
+        self.assertFalse(wt.has_or_had_id('b-id'))
+        self.assertFalse(wt.has_or_had_id('c-id'))
+        self.assertFalse(wt.has_or_had_id('d-id'))
+        self.assertTrue(wt.has_filename('a'))
+        self.assertFalse(wt.has_filename('b'))
+        self.assertFalse(wt.has_filename('b/c'))
+        self.assertFalse(wt.has_filename('d'))
+        wt.unlock()
+        # the changes should have persisted to disk - reopen the workingtree
+        # to be sure.
+        wt = wt.bzrdir.open_workingtree()
+        wt.lock_read()
+        self.assertTrue(wt.has_id('a-id'))
+        self.assertFalse(wt.has_or_had_id('b-id'))
+        self.assertFalse(wt.has_or_had_id('c-id'))
+        self.assertFalse(wt.has_or_had_id('d-id'))
+        self.assertTrue(wt.has_filename('a'))
+        self.assertFalse(wt.has_filename('b'))
+        self.assertFalse(wt.has_filename('b/c'))
+        self.assertFalse(wt.has_filename('d'))
+        wt.unlock()
+        
+
 class TestCommitProgress(TestCaseWithWorkingTree):
     
     def restoreDefaults(self):

=== modified file 'bzrlib/tests/workingtree_implementations/test_parents.py'
--- bzrlib/tests/workingtree_implementations/test_parents.py	2006-09-05 23:08:03 +0000
+++ bzrlib/tests/workingtree_implementations/test_parents.py	2006-09-08 00:19:48 +0000
@@ -33,13 +33,9 @@
         """
         self.assertEqual(expected, tree.get_parent_ids())
         if expected == []:
-            self.assertEqual(None,
-                self.applyDeprecated(symbol_versioning.zero_eleven,
-                    tree.last_revision))
+            self.assertEqual(None, tree.last_revision())
         else:
-            self.assertEqual(expected[0],
-                self.applyDeprecated(symbol_versioning.zero_eleven,
-                    tree.last_revision))
+            self.assertEqual(expected[0], tree.last_revision())
         self.assertEqual(expected[1:],
             self.applyDeprecated(symbol_versioning.zero_eleven,
                 tree.pending_merges))

=== added file 'bzrlib/tests/workingtree_implementations/test_unversion.py'
--- bzrlib/tests/workingtree_implementations/test_unversion.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_unversion.py	2006-09-07 08:13:22 +0000
@@ -0,0 +1,76 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests of the WorkingTree.unversion API."""
+
+from bzrlib import errors
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+
+
+class TestUnversion(TestCaseWithWorkingTree):
+
+    def test_unversion_requires_write_lock(self):
+        """WT.unversion([]) in a read lock raises ReadOnlyError."""
+        tree = self.make_branch_and_tree('.')
+        tree.lock_read()
+        self.assertRaises(errors.ReadOnlyError, tree.unversion, [])
+        tree.unlock()
+
+    def test_unversion_missing_file(self):
+        """WT.unversion(['missing-id']) raises NoSuchId."""
+        tree = self.make_branch_and_tree('.')
+        self.assertRaises(errors.NoSuchId, tree.unversion, ['missing-id'])
+
+    def test_unversion_several_files(self):
+        """After unversioning several files, they should not be versioned."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b', 'c'])
+        tree.add(['a', 'b', 'c'], ['a-id', 'b-id', 'c-id'])
+        # within a lock unversion should take effect
+        tree.lock_write()
+        tree.unversion(['a-id', 'b-id'])
+        self.assertFalse(tree.has_id('a-id'))
+        self.assertFalse(tree.has_id('b-id'))
+        self.assertTrue(tree.has_id('c-id'))
+        self.assertTrue(tree.has_filename('a'))
+        self.assertTrue(tree.has_filename('b'))
+        self.assertTrue(tree.has_filename('c'))
+        tree.unlock()
+        # the changes should have persisted to disk - reopen the workingtree
+        # to be sure.
+        tree = tree.bzrdir.open_workingtree()
+        self.assertFalse(tree.has_id('a-id'))
+        self.assertFalse(tree.has_id('b-id'))
+        self.assertTrue(tree.has_id('c-id'))
+        self.assertTrue(tree.has_filename('a'))
+        self.assertTrue(tree.has_filename('b'))
+        self.assertTrue(tree.has_filename('c'))
+        
+    def test_unversion_subtree(self):
+        """Unversioning the root of a subtree unversions the entire subtree."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a/', 'a/b', 'c'])
+        tree.add(['a', 'a/b', 'c'], ['a-id', 'b-id', 'c-id'])
+        # within a lock unversion should take effect
+        tree.lock_write()
+        tree.unversion(['a-id'])
+        self.assertFalse(tree.has_id('a-id'))
+        self.assertFalse(tree.has_id('b-id'))
+        self.assertTrue(tree.has_id('c-id'))
+        self.assertTrue(tree.has_filename('a'))
+        self.assertTrue(tree.has_filename('a/b'))
+        self.assertTrue(tree.has_filename('c'))
+        tree.unlock()

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-09-05 08:14:23 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-09-08 00:19:48 +0000
@@ -19,7 +19,6 @@
 import os
 import sys
 
-from bzrlib import ignores
 import bzrlib
 from bzrlib import branch, bzrdir, errors, osutils, urlutils, workingtree
 from bzrlib.errors import (NotBranchError, NotVersionedError,
@@ -49,14 +48,12 @@
 
     def test_list_files_sorted(self):
         tree = self.make_branch_and_tree('.')
-        ignores._set_user_ignores(['./.bazaar'])
         self.build_tree(['dir/', 'file', 'dir/file', 'dir/b',
                          'dir/subdir/', 'a', 'dir/subfile',
                          'zz_dir/', 'zz_dir/subfile'])
         files = [(path, kind) for (path, v, kind, file_id, entry)
                                in tree.list_files()]
         self.assertEqual([
-            ('.bazaar', 'directory'),
             ('a', 'file'),
             ('dir', 'directory'),
             ('file', 'file'),
@@ -67,7 +64,6 @@
         files = [(path, kind) for (path, v, kind, file_id, entry)
                                in tree.list_files()]
         self.assertEqual([
-            ('.bazaar', 'directory'),
             ('a', 'file'),
             ('dir', 'directory'),
             ('dir/b', 'file'),

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-08-31 06:06:38 +0000
+++ bzrlib/transform.py	2006-09-06 02:47:54 +0000
@@ -1326,10 +1326,11 @@
             trans_id = conflict[1]
             try:
                 tt.cancel_deletion(trans_id)
-                new_conflicts.add((c_type, 'Not deleting', trans_id))
+                new_conflicts.add(('deleting parent', 'Not deleting', 
+                                   trans_id))
             except KeyError:
                 tt.create_directory(trans_id)
-                new_conflicts.add((c_type, 'Created directory.', trans_id))
+                new_conflicts.add((c_type, 'Created directory', trans_id))
         elif c_type == 'unversioned parent':
             tt.version_file(tt.inactive_file_id(conflict[1]), conflict[1])
             new_conflicts.add((c_type, 'Versioned directory', conflict[1]))

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-08-30 15:33:53 +0000
+++ bzrlib/transport/__init__.py	2006-09-06 20:42:45 +0000
@@ -29,6 +29,7 @@
 import errno
 from collections import deque
 from copy import deepcopy
+from cStringIO import StringIO
 import re
 from stat import S_ISDIR
 import sys
@@ -38,15 +39,23 @@
 import warnings
 
 import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    osutils,
+    symbol_versioning,
+    urlutils,
+    )
 from bzrlib.errors import DependencyNotPresent
-import bzrlib.osutils as osutils
 from bzrlib.osutils import pumpfile
-from bzrlib.symbol_versioning import (deprecated_passed, deprecated_method, deprecated_function, 
+from bzrlib.symbol_versioning import (
+        deprecated_passed,
+        deprecated_method,
+        deprecated_function,
         DEPRECATED_PARAMETER,
-        zero_eight)
+        zero_eight,
+        zero_eleven,
+        )
 from bzrlib.trace import mutter, warning
-import bzrlib.urlutils as urlutils
 
 # {prefix: [transport_classes]}
 # Transports are inserted onto the list LIFO and tried in order; as a result
@@ -248,10 +257,9 @@
         object or string to another one.
         This just gives them something easy to call.
         """
-        if isinstance(from_file, basestring):
-            to_file.write(from_file)
-        else:
-            pumpfile(from_file, to_file)
+        assert not isinstance(from_file, basestring), \
+            '_pump should only be called on files not %s' % (type(from_file,))
+        pumpfile(from_file, to_file)
 
     def _get_total(self, multi):
         """Try to figure out how many entries are in multi,
@@ -373,6 +381,13 @@
         """
         raise NotImplementedError(self.get)
 
+    def get_bytes(self, relpath):
+        """Get a raw string of the bytes for a file at the given location.
+
+        :param relpath: The relative path to the file
+        """
+        return self.get(relpath).read()
+
     def readv(self, relpath, offsets):
         """Get parts of the file at the given relative path.
 
@@ -483,16 +498,106 @@
             yield self.get(relpath)
             count += 1
 
+    @deprecated_method(zero_eleven)
     def put(self, relpath, f, mode=None):
-        """Copy the file-like or string object into the location.
+        """Copy the file-like object into the location.
 
         :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
+        :param f:       File-like object.
         :param mode: The mode for the newly created file, 
                      None means just use the default
         """
-        raise NotImplementedError(self.put)
-
+        if isinstance(f, str):
+            return self.put_bytes(relpath, f, mode=mode)
+        else:
+            return self.put_file(relpath, f, mode=mode)
+
+    def put_bytes(self, relpath, bytes, mode=None):
+        """Atomically put the supplied bytes into the given location.
+
+        :param relpath: The location to put the contents, relative to the
+            transport base.
+        :param bytes: A bytestring of data.
+        :param mode: Create the file with the given mode.
+        :return: None
+        """
+        assert isinstance(bytes, str), \
+            'bytes must be a plain string, not %s' % type(bytes)
+        return self.put_file(relpath, StringIO(bytes), mode=mode)
+
+    def put_bytes_non_atomic(self, relpath, bytes, mode=None,
+                             create_parent_dir=False,
+                             dir_mode=None):
+        """Copy the string into the target location.
+
+        This function is not strictly safe to use. See 
+        Transport.put_bytes_non_atomic for more information.
+
+        :param relpath: The remote location to put the contents.
+        :param bytes:   A string object containing the raw bytes to write into
+                        the target file.
+        :param mode:    Possible access permissions for new file.
+                        None means do not set remote permissions.
+        :param create_parent_dir: If we cannot create the target file because
+                        the parent directory does not exist, go ahead and
+                        create it, and then try again.
+        :param dir_mode: Possible access permissions for new directories.
+        """
+        assert isinstance(bytes, str), \
+            'bytes must be a plain string, not %s' % type(bytes)
+        self.put_file_non_atomic(relpath, StringIO(bytes), mode=mode,
+                                 create_parent_dir=create_parent_dir,
+                                 dir_mode=dir_mode)
+
+    def put_file(self, relpath, f, mode=None):
+        """Copy the file-like object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like object.
+        :param mode: The mode for the newly created file,
+                     None means just use the default.
+        """
+        # We would like to mark this as NotImplemented, but most likely
+        # transports have defined it in terms of the old api.
+        symbol_versioning.warn('Transport %s should implement put_file,'
+                               ' rather than implementing put() as of'
+                               ' version 0.11.'
+                               % (self.__class__.__name__,),
+                               DeprecationWarning)
+        return self.put(relpath, f, mode=mode)
+        #raise NotImplementedError(self.put_file)
+
+    def put_file_non_atomic(self, relpath, f, mode=None,
+                            create_parent_dir=False,
+                            dir_mode=None):
+        """Copy the file-like object into the target location.
+
+        This function is not strictly safe to use. It is only meant to
+        be used when you already know that the target does not exist.
+        It is not safe, because it will open and truncate the remote
+        file. So there may be a time when the file has invalid contents.
+
+        :param relpath: The remote location to put the contents.
+        :param f:       File-like object.
+        :param mode:    Possible access permissions for new file.
+                        None means do not set remote permissions.
+        :param create_parent_dir: If we cannot create the target file because
+                        the parent directory does not exist, go ahead and
+                        create it, and then try again.
+        :param dir_mode: Possible access permissions for new directories.
+        """
+        # Default implementation just does an atomic put.
+        try:
+            return self.put_file(relpath, f, mode=mode)
+        except errors.NoSuchFile:
+            if not create_parent_dir:
+                raise
+            parent_dir = osutils.dirname(relpath)
+            if parent_dir:
+                self.mkdir(parent_dir, mode=dir_mode)
+                return self.put_file(relpath, f, mode=mode)
+
+    @deprecated_method(zero_eleven)
     def put_multi(self, files, mode=None, pb=None):
         """Put a set of files into the location.
 
@@ -501,9 +606,12 @@
         :param mode: The mode for the newly created files
         :return: The number of files copied.
         """
-        def put(path, f):
-            self.put(path, f, mode=mode)
-        return len(self._iterate_over(files, put, pb, 'put', expand=True))
+        def _put(path, f):
+            if isinstance(f, str):
+                self.put_bytes(path, f, mode=mode)
+            else:
+                self.put_file(path, f, mode=mode)
+        return len(self._iterate_over(files, _put, pb, 'put', expand=True))
 
     def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
@@ -515,15 +623,40 @@
             self.mkdir(path, mode=mode)
         return len(self._iterate_over(relpaths, mkdir, pb, 'mkdir', expand=False))
 
+    @deprecated_method(zero_eleven)
     def append(self, relpath, f, mode=None):
-        """Append the text in the file-like or string object to 
-        the supplied location.
-
-        returns the length of f before the content was written to it.
-        
-        If the file does not exist, it is created with the supplied mode.
-        """
-        raise NotImplementedError(self.append)
+        """Append the text in the file-like object to the supplied location.
+
+        returns the length of relpath before the content was written to it.
+        
+        If the file does not exist, it is created with the supplied mode.
+        """
+        return self.append_file(relpath, f, mode=mode)
+
+    def append_file(self, relpath, f, mode=None):
+        """Append the text in the file-like object to the supplied location.
+
+        returns the length of relpath before the content was written to it.
+        
+        If the file does not exist, it is created with the supplied mode.
+        """
+        symbol_versioning.warn('Transport %s should implement append_file,'
+                               ' rather than implementing append() as of'
+                               ' version 0.11.'
+                               % (self.__class__.__name__,),
+                               DeprecationWarning)
+        return self.append(relpath, f, mode=mode)
+
+    def append_bytes(self, relpath, bytes, mode=None):
+        """Append the text in the string object to the supplied location.
+
+        returns the length of relpath before the content was written to it.
+        
+        If the file does not exist, it is created with the supplied mode.
+        """
+        assert isinstance(bytes, str), \
+            'bytes must be a plain string, not %s' % type(bytes)
+        return self.append_file(relpath, StringIO(bytes), mode=mode)
 
     def append_multi(self, files, pb=None):
         """Append the text in each file-like or string object to
@@ -532,7 +665,7 @@
         :param files: A set of (path, f) entries
         :param pb:  An optional ProgressBar for indicating percent done.
         """
-        return self._iterate_over(files, self.append, pb, 'append', expand=True)
+        return self._iterate_over(files, self.append_file, pb, 'append', expand=True)
 
     def copy(self, rel_from, rel_to):
         """Copy the item at rel_from to the location at rel_to.
@@ -540,7 +673,7 @@
         Override this for efficiency if a specific transport can do it 
         faster than this default implementation.
         """
-        self.put(rel_to, self.get(rel_from))
+        self.put_file(rel_to, self.get(rel_from))
 
     def copy_multi(self, relpaths, pb=None):
         """Copy a bunch of entries.
@@ -561,7 +694,7 @@
         """
         # The dummy implementation just does a simple get + put
         def copy_entry(path):
-            other.put(path, self.get(path), mode=mode)
+            other.put_file(path, self.get(path), mode=mode)
 
         return len(self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False))
 

=== modified file 'bzrlib/transport/decorator.py'
--- bzrlib/transport/decorator.py	2006-07-28 16:05:23 +0000
+++ bzrlib/transport/decorator.py	2006-09-05 21:20:24 +0000
@@ -57,9 +57,13 @@
         """See Transport.abspath()."""
         return self._get_url_prefix() + self._decorated.abspath(relpath)
 
-    def append(self, relpath, f, mode=None):
-        """See Transport.append()."""
-        return self._decorated.append(relpath, f, mode=mode)
+    def append_file(self, relpath, f, mode=None):
+        """See Transport.append_file()."""
+        return self._decorated.append_file(relpath, f, mode=mode)
+
+    def append_bytes(self, relpath, bytes, mode=None):
+        """See Transport.append_bytes()."""
+        return self._decorated.append_bytes(relpath, bytes, mode=mode)
 
     def clone(self, offset=None):
         """See Transport.clone()."""
@@ -96,10 +100,14 @@
         """See Transport.mkdir()."""
         return self._decorated.mkdir(relpath, mode)
 
-    def put(self, relpath, f, mode=None):
-        """See Transport.put()."""
-        return self._decorated.put(relpath, f, mode)
+    def put_file(self, relpath, f, mode=None):
+        """See Transport.put_file()."""
+        return self._decorated.put_file(relpath, f, mode)
     
+    def put_bytes(self, relpath, bytes, mode=None):
+        """See Transport.put_bytes()."""
+        return self._decorated.put_bytes(relpath, bytes, mode)
+
     def listable(self):
         """See Transport.listable."""
         return self._decorated.listable()

=== modified file 'bzrlib/transport/fakevfat.py'
--- bzrlib/transport/fakevfat.py	2006-07-28 16:05:23 +0000
+++ bzrlib/transport/fakevfat.py	2006-09-05 19:34:34 +0000
@@ -91,8 +91,8 @@
     def readv(self, relpath, offsets):
         return self._decorated.readv(self._squash_name(relpath), offsets)
 
-    def put(self, relpath, f, mode=None):
-        return self._decorated.put(self._squash_name(relpath), f, mode)
+    def put_file(self, relpath, f, mode=None):
+        return self._decorated.put_file(self._squash_name(relpath), f, mode)
 
 
 class FakeVFATServer(DecoratorServer):

=== modified file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	2006-08-25 22:41:03 +0000
+++ bzrlib/transport/ftp.py	2006-09-05 21:20:24 +0000
@@ -294,7 +294,7 @@
                 self._FTP_instance = None
                 return self.get(relpath, decode, retries+1)
 
-    def put(self, relpath, fp, mode=None, retries=0):
+    def put_file(self, relpath, fp, mode=None, retries=0):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
@@ -333,7 +333,7 @@
             else:
                 warning("FTP temporary error: %s. Retrying.", str(e))
                 self._FTP_instance = None
-                self.put(relpath, fp, mode, retries+1)
+                self.put_file(relpath, fp, mode, retries+1)
         except EOFError:
             if retries > _number_of_retries:
                 raise errors.TransportError("FTP control connection closed during PUT %s."
@@ -342,7 +342,7 @@
                 warning("FTP control connection closed. Trying to reopen.")
                 time.sleep(_sleep_between_retries)
                 self._FTP_instance = None
-                self.put(relpath, fp, mode, retries+1)
+                self.put_file(relpath, fp, mode, retries+1)
 
     def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
@@ -365,7 +365,7 @@
         except ftplib.error_perm, e:
             self._translate_perm_error(e, abspath, unknown_exc=errors.PathError)
 
-    def append(self, relpath, f, mode=None):
+    def append_file(self, relpath, f, mode=None):
         """Append the text in the file-like object into the final
         location.
         """

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2006-08-31 10:33:01 +0000
+++ bzrlib/transport/http/__init__.py	2006-09-05 21:20:24 +0000
@@ -284,11 +284,11 @@
 
         return combined
 
-    def put(self, relpath, f, mode=None):
-        """Copy the file-like or string object into the location.
+    def put_file(self, relpath, f, mode=None):
+        """Copy the file-like object into the location.
 
         :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
+        :param f:       File-like object.
         """
         raise TransportNotPossible('http PUT not supported')
 
@@ -300,7 +300,7 @@
         """See Transport.rmdir."""
         raise TransportNotPossible('http does not support rmdir()')
 
-    def append(self, relpath, f):
+    def append_file(self, relpath, f, mode=None):
         """Append the text in the file-like object into the final
         location.
         """

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2006-08-25 22:19:48 +0000
+++ bzrlib/transport/local.py	2006-09-06 20:42:45 +0000
@@ -19,6 +19,7 @@
 This is a fairly thin wrapper on regular file IO.
 """
 
+import errno
 import os
 import shutil
 import sys
@@ -26,6 +27,7 @@
 import tempfile
 
 from bzrlib import (
+    atomicfile,
     osutils,
     urlutils,
     )
@@ -37,6 +39,7 @@
 
 
 _append_flags = os.O_CREAT | os.O_APPEND | os.O_WRONLY | osutils.O_BINARY
+_put_non_atomic_flags = os.O_CREAT | os.O_TRUNC | os.O_WRONLY | osutils.O_BINARY
 
 
 class LocalTransport(Transport):
@@ -130,19 +133,20 @@
         except (IOError, OSError),e:
             self._translate_error(e, path)
 
-    def put(self, relpath, f, mode=None):
-        """Copy the file-like or string object into the location.
+    def put_file(self, relpath, f, mode=None):
+        """Copy the file-like object into the location.
 
         :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
+        :param f:       File-like object.
+        :param mode: The mode for the newly created file, 
+                     None means just use the default
         """
-        from bzrlib.atomicfile import AtomicFile
 
         path = relpath
         try:
             path = self._abspath(relpath)
             check_legal_path(path)
-            fp = AtomicFile(path, 'wb', new_mode=mode)
+            fp = atomicfile.AtomicFile(path, 'wb', new_mode=mode)
         except (IOError, OSError),e:
             self._translate_error(e, path)
         try:
@@ -151,6 +155,107 @@
         finally:
             fp.close()
 
+    def put_bytes(self, relpath, bytes, mode=None):
+        """Copy the string into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param bytes:   String
+        """
+
+        path = relpath
+        try:
+            path = self._abspath(relpath)
+            check_legal_path(path)
+            fp = atomicfile.AtomicFile(path, 'wb', new_mode=mode)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
+        try:
+            fp.write(bytes)
+            fp.commit()
+        finally:
+            fp.close()
+
+    def _put_non_atomic_helper(self, relpath, writer,
+                               mode=None,
+                               create_parent_dir=False,
+                               dir_mode=None):
+        """Common functionality information for the put_*_non_atomic.
+
+        This tracks all the create_parent_dir stuff.
+
+        :param relpath: the path we are putting to.
+        :param writer: A function that takes an os level file descriptor
+            and writes whatever data it needs to write there.
+        :param mode: The final file mode.
+        :param create_parent_dir: Should we be creating the parent directory
+            if it doesn't exist?
+        """
+        abspath = self._abspath(relpath)
+        if mode is None:
+            # os.open() will automatically use the umask
+            local_mode = 0666
+        else:
+            local_mode = mode
+        try:
+            fd = os.open(abspath, _put_non_atomic_flags, local_mode)
+        except (IOError, OSError),e:
+            # We couldn't create the file, maybe we need to create
+            # the parent directory, and try again
+            if (not create_parent_dir
+                or e.errno not in (errno.ENOENT,errno.ENOTDIR)):
+                self._translate_error(e, relpath)
+            parent_dir = os.path.dirname(abspath)
+            if not parent_dir:
+                self._translate_error(e, relpath)
+            self._mkdir(parent_dir, mode=dir_mode)
+            # We created the parent directory, lets try to open the
+            # file again
+            try:
+                fd = os.open(abspath, _put_non_atomic_flags, local_mode)
+            except (IOError, OSError), e:
+                self._translate_error(e, relpath)
+        try:
+            st = os.fstat(fd)
+            if mode is not None and mode != S_IMODE(st.st_mode):
+                # Because of umask, we may still need to chmod the file.
+                # But in the general case, we won't have to
+                os.chmod(abspath, mode)
+            writer(fd)
+        finally:
+            os.close(fd)
+
+    def put_file_non_atomic(self, relpath, f, mode=None,
+                            create_parent_dir=False,
+                            dir_mode=None):
+        """Copy the file-like object into the target location.
+
+        This function is not strictly safe to use. It is only meant to
+        be used when you already know that the target does not exist.
+        It is not safe, because it will open and truncate the remote
+        file. So there may be a time when the file has invalid contents.
+
+        :param relpath: The remote location to put the contents.
+        :param f:       File-like object.
+        :param mode:    Possible access permissions for new file.
+                        None means do not set remote permissions.
+        :param create_parent_dir: If we cannot create the target file because
+                        the parent directory does not exist, go ahead and
+                        create it, and then try again.
+        """
+        def writer(fd):
+            self._pump_to_fd(f, fd)
+        self._put_non_atomic_helper(relpath, writer, mode=mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
+
+    def put_bytes_non_atomic(self, relpath, bytes, mode=None,
+                             create_parent_dir=False, dir_mode=None):
+        def writer(fd):
+            os.write(fd, bytes)
+        self._put_non_atomic_helper(relpath, writer, mode=mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
+
     def iter_files_recursive(self):
         """Iter the relative paths of files in the transports sub-tree."""
         queue = list(self.list_dir(u'.'))
@@ -163,48 +268,68 @@
             else:
                 yield relpath
 
-    def mkdir(self, relpath, mode=None):
-        """Create a directory at the given path."""
-        path = relpath
+    def _mkdir(self, abspath, mode=None):
+        """Create a real directory, filtering through mode"""
+        if mode is None:
+            # os.mkdir() will filter through umask
+            local_mode = 0777
+        else:
+            local_mode = mode
         try:
-            if mode is None:
-                # os.mkdir() will filter through umask
-                local_mode = 0777
-            else:
-                local_mode = mode
-            path = self._abspath(relpath)
-            os.mkdir(path, local_mode)
+            os.mkdir(abspath, local_mode)
             if mode is not None:
                 # It is probably faster to just do the chmod, rather than
                 # doing a stat, and then trying to compare
-                os.chmod(path, mode)
-        except (IOError, OSError),e:
-            self._translate_error(e, path)
-
-    def append(self, relpath, f, mode=None):
+                os.chmod(abspath, mode)
+        except (IOError, OSError),e:
+            self._translate_error(e, abspath)
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        self._mkdir(self._abspath(relpath), mode=mode)
+
+    def _get_append_file(self, relpath, mode=None):
+        """Call os.open() for the given relpath"""
+        file_abspath = self._abspath(relpath)
+        if mode is None:
+            # os.open() will automatically use the umask
+            local_mode = 0666
+        else:
+            local_mode = mode
+        try:
+            return file_abspath, os.open(file_abspath, _append_flags, local_mode)
+        except (IOError, OSError),e:
+            self._translate_error(e, relpath)
+
+    def _check_mode_and_size(self, file_abspath, fd, mode=None):
+        """Check the mode of the file, and return the current size"""
+        st = os.fstat(fd)
+        if mode is not None and mode != S_IMODE(st.st_mode):
+            # Because of umask, we may still need to chmod the file.
+            # But in the general case, we won't have to
+            os.chmod(file_abspath, mode)
+        return st.st_size
+
+    def append_file(self, relpath, f, mode=None):
         """Append the text in the file-like object into the final location."""
-        abspath = self._abspath(relpath)
-        if mode is None:
-            # os.open() will automatically use the umask
-            local_mode = 0666
-        else:
-            local_mode = mode
-        try:
-            fd = os.open(abspath, _append_flags, local_mode)
-        except (IOError, OSError),e:
-            self._translate_error(e, relpath)
-        try:
-            st = os.fstat(fd)
-            result = st.st_size
-            if mode is not None and mode != S_IMODE(st.st_mode):
-                # Because of umask, we may still need to chmod the file.
-                # But in the general case, we won't have to
-                os.chmod(abspath, mode)
+        file_abspath, fd = self._get_append_file(relpath, mode=mode)
+        try:
+            result = self._check_mode_and_size(file_abspath, fd, mode=mode)
             self._pump_to_fd(f, fd)
         finally:
             os.close(fd)
         return result
 
+    def append_bytes(self, relpath, bytes, mode=None):
+        """Append the text in the string into the final location."""
+        file_abspath, fd = self._get_append_file(relpath, mode=mode)
+        try:
+            result = self._check_mode_and_size(file_abspath, fd, mode=mode)
+            os.write(fd, bytes)
+        finally:
+            os.close(fd)
+        return result
+
     def _pump_to_fd(self, fromfile, to_fd):
         """Copy contents of one file to another."""
         BUFSIZE = 32768

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2006-08-30 14:37:31 +0000
+++ bzrlib/transport/memory.py	2006-09-05 21:25:15 +0000
@@ -98,8 +98,8 @@
         else:
             return temp_t.base[:-1]
 
-    def append(self, relpath, f, mode=None):
-        """See Transport.append()."""
+    def append_file(self, relpath, f, mode=None):
+        """See Transport.append_file()."""
         _abspath = self._abspath(relpath)
         self._check_parent(_abspath)
         orig_content, orig_mode = self._files.get(_abspath, ("", None))
@@ -133,8 +133,8 @@
             raise NoSuchFile(relpath)
         return StringIO(self._files[_abspath][0])
 
-    def put(self, relpath, f, mode=None):
-        """See Transport.put()."""
+    def put_file(self, relpath, f, mode=None):
+        """See Transport.put_file()."""
         _abspath = self._abspath(relpath)
         self._check_parent(_abspath)
         self._files[_abspath] = (f.read(), mode)

=== modified file 'bzrlib/transport/readonly.py'
--- bzrlib/transport/readonly.py	2006-07-28 16:05:23 +0000
+++ bzrlib/transport/readonly.py	2006-09-05 21:20:24 +0000
@@ -26,8 +26,12 @@
     This is requested via the 'readonly+' prefix to get_transport().
     """
 
-    def append(self, relpath, f):
-        """See Transport.append()."""
+    def append_file(self, relpath, f, mode=None):
+        """See Transport.append_file()."""
+        raise TransportNotPossible('readonly transport')
+    
+    def append_bytes(self, relpath, bytes, mode=None):
+        """See Transport.append_bytes()."""
         raise TransportNotPossible('readonly transport')
     
     @classmethod
@@ -43,8 +47,12 @@
         """See Transport.delete_tree()."""
         raise TransportNotPossible('readonly transport')
 
-    def put(self, relpath, f, mode=None):
-        """See Transport.put()."""
+    def put_file(self, relpath, f, mode=None):
+        """See Transport.put_file()."""
+        raise TransportNotPossible('readonly transport')
+
+    def put_bytes(self, relpath, bytes, mode=None):
+        """See Transport.put_bytes()."""
         raise TransportNotPossible('readonly transport')
 
     def mkdir(self, relpath, mode=None):

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-08-30 14:37:31 +0000
+++ bzrlib/transport/sftp.py	2006-09-06 21:51:39 +0000
@@ -366,12 +366,12 @@
             # on to the next
             cur_coalesced = cur_coalesced_stack.next()
 
-    def put(self, relpath, f, mode=None):
+    def put_file(self, relpath, f, mode=None):
         """
-        Copy the file-like or string object into the location.
+        Copy the file-like object into the location.
 
         :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
+        :param f:       File-like object.
         :param mode: The final mode for the file
         """
         final_path = self._remote_path(relpath)
@@ -389,6 +389,19 @@
                 self._pump(f, fout)
             except (IOError, paramiko.SSHException), e:
                 self._translate_io_exception(e, tmp_abspath)
+            # XXX: This doesn't truly help like we would like it to.
+            #      The problem is that openssh strips sticky bits. So while we
+            #      can properly set group write permission, we lose the group
+            #      sticky bit. So it is probably best to stop chmodding, and
+            #      just tell users that they need to set the umask correctly.
+            #      The attr.st_mode = mode, in _sftp_open_exclusive
+            #      will handle when the user wants the final mode to be more 
+            #      restrictive. And then we avoid a round trip. Unless 
+            #      paramiko decides to expose an async chmod()
+
+            # This is designed to chmod() right before we close.
+            # Because we set_pipelined() earlier, theoretically we might 
+            # avoid the round trip for fout.close()
             if mode is not None:
                 self._sftp.chmod(tmp_abspath, mode)
             fout.close()
@@ -412,6 +425,83 @@
             # raise the original with its traceback if we can.
             raise
 
+    def _put_non_atomic_helper(self, relpath, writer, mode=None,
+                               create_parent_dir=False,
+                               dir_mode=None):
+        abspath = self._remote_path(relpath)
+
+        # TODO: jam 20060816 paramiko doesn't publicly expose a way to
+        #       set the file mode at create time. If it does, use it.
+        #       But for now, we just chmod later anyway.
+
+        def _open_and_write_file():
+            """Try to open the target file, raise error on failure"""
+            fout = None
+            try:
+                try:
+                    fout = self._sftp.file(abspath, mode='wb')
+                    fout.set_pipelined(True)
+                    writer(fout)
+                except (paramiko.SSHException, IOError), e:
+                    self._translate_io_exception(e, abspath,
+                                                 ': unable to open')
+
+                # This is designed to chmod() right before we close.
+                # Because we set_pipelined() earlier, theoretically we might 
+                # avoid the round trip for fout.close()
+                if mode is not None:
+                    self._sftp.chmod(abspath, mode)
+            finally:
+                if fout is not None:
+                    fout.close()
+
+        if not create_parent_dir:
+            _open_and_write_file()
+            return
+
+        # Try error handling to create the parent directory if we need to
+        try:
+            _open_and_write_file()
+        except NoSuchFile:
+            # Try to create the parent directory, and then go back to
+            # writing the file
+            parent_dir = os.path.dirname(abspath)
+            self._mkdir(parent_dir, dir_mode)
+            _open_and_write_file()
+
+    def put_file_non_atomic(self, relpath, f, mode=None,
+                            create_parent_dir=False,
+                            dir_mode=None):
+        """Copy the file-like object into the target location.
+
+        This function is not strictly safe to use. It is only meant to
+        be used when you already know that the target does not exist.
+        It is not safe, because it will open and truncate the remote
+        file. So there may be a time when the file has invalid contents.
+
+        :param relpath: The remote location to put the contents.
+        :param f:       File-like object.
+        :param mode:    Possible access permissions for new file.
+                        None means do not set remote permissions.
+        :param create_parent_dir: If we cannot create the target file because
+                        the parent directory does not exist, go ahead and
+                        create it, and then try again.
+        """
+        def writer(fout):
+            self._pump(f, fout)
+        self._put_non_atomic_helper(relpath, writer, mode=mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
+
+    def put_bytes_non_atomic(self, relpath, bytes, mode=None,
+                             create_parent_dir=False,
+                             dir_mode=None):
+        def writer(fout):
+            fout.write(bytes)
+        self._put_non_atomic_helper(relpath, writer, mode=mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
+
     def iter_files_recursive(self):
         """Walk the relative paths of all files in this transport."""
         queue = list(self.list_dir('.'))
@@ -424,21 +514,22 @@
             else:
                 yield relpath
 
+    def _mkdir(self, abspath, mode=None):
+        if mode is None:
+            local_mode = 0777
+        else:
+            local_mode = mode
+        try:
+            self._sftp.mkdir(abspath, local_mode)
+            if mode is not None:
+                self._sftp.chmod(abspath, mode=mode)
+        except (paramiko.SSHException, IOError), e:
+            self._translate_io_exception(e, abspath, ': unable to mkdir',
+                failure_exc=FileExists)
+
     def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
-        path = self._remote_path(relpath)
-        try:
-            # In the paramiko documentation, it says that passing a mode flag 
-            # will filtered against the server umask.
-            # StubSFTPServer does not do this, which would be nice, because it is
-            # what we really want :)
-            # However, real servers do use umask, so we really should do it that way
-            self._sftp.mkdir(path)
-            if mode is not None:
-                self._sftp.chmod(path, mode=mode)
-        except (paramiko.SSHException, IOError), e:
-            self._translate_io_exception(e, path, ': unable to mkdir',
-                failure_exc=FileExists)
+        self._mkdir(self._remote_path(relpath), mode=mode)
 
     def _translate_io_exception(self, e, path, more_info='', 
                                 failure_exc=PathError):
@@ -470,7 +561,7 @@
             mutter('Raising exception with errno %s', e.errno)
         raise e
 
-    def append(self, relpath, f, mode=None):
+    def append_file(self, relpath, f, mode=None):
         """
         Append the text in the file-like object into the final
         location.
@@ -641,6 +732,10 @@
         :param abspath: The remote absolute path where the file should be opened
         :param mode: The mode permissions bits for the new file
         """
+        # TODO: jam 20060816 Paramiko >= 1.6.2 (probably earlier) supports
+        #       using the 'x' flag to indicate SFTP_FLAG_EXCL.
+        #       However, there is no way to set the permission mode at open 
+        #       time using the sftp_client.file() functionality.
         path = self._sftp._adjust_cwd(abspath)
         # mutter('sftp abspath %s => %s', abspath, path)
         attr = SFTPAttributes()
@@ -658,6 +753,12 @@
             self._translate_io_exception(e, abspath, ': unable to open',
                 failure_exc=FileExists)
 
+    def _can_roundtrip_unix_modebits(self):
+        if sys.platform == 'win32':
+            # anyone else?
+            return False
+        else:
+            return True
 
 # ------------- server test implementation --------------
 import threading

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-08-30 13:54:51 +0000
+++ bzrlib/tree.py	2006-09-08 01:58:47 +0000
@@ -169,6 +169,21 @@
     def lock_read(self):
         pass
 
+    def revision_tree(self, revision_id):
+        """Obtain a revision tree for the revision revision_id.
+
+        The intention of this method is to allow access to possibly cached
+        tree data. Implementors of this method should raise NoSuchRevision if
+        the tree is not locally available, even if they could obtain the 
+        tree via a repository or some other means. Callers are responsible 
+        for finding the ultimate source for a revision tree.
+
+        :param revision_id: The revision_id of the requested tree.
+        :return: A Tree.
+        :raises: NoSuchRevision if the tree cannot be obtained.
+        """
+        raise errors.NoSuchRevisionInTree(self, revision_id)
+
     def unknowns(self):
         """What files are present in this tree and unknown.
         

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-08-30 13:54:51 +0000
+++ bzrlib/weave.py	2006-09-05 20:08:52 +0000
@@ -1132,7 +1132,7 @@
         sio = StringIO()
         write_weave_v5(self, sio)
         sio.seek(0)
-        transport.put(name + WeaveFile.WEAVE_SUFFIX, sio, self._filemode)
+        transport.put_file(name + WeaveFile.WEAVE_SUFFIX, sio, self._filemode)
 
     def create_empty(self, name, transport, filemode=None):
         return WeaveFile(name, transport, filemode, create=True)
@@ -1143,9 +1143,9 @@
         sio = StringIO()
         write_weave_v5(self, sio)
         sio.seek(0)
-        self._transport.put(self._weave_name + WeaveFile.WEAVE_SUFFIX,
-                            sio,
-                            self._filemode)
+        self._transport.put_file(self._weave_name + WeaveFile.WEAVE_SUFFIX,
+                                 sio,
+                                 self._filemode)
 
     @staticmethod
     def get_suffixes():

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-09-05 23:15:30 +0000
+++ bzrlib/workingtree.py	2006-09-08 00:19:48 +0000
@@ -566,7 +566,6 @@
         args = (DEPRECATED_PARAMETER, message, ) + args
         committed_id = Commit().commit( working_tree=self, revprops=revprops,
             *args, **kwargs)
-        self._set_inventory(self.read_working_inventory())
         return committed_id
 
     def id2abspath(self, file_id):
@@ -1105,7 +1104,33 @@
         for subp in self.extras():
             if not self.is_ignored(subp):
                 yield subp
-
+    
+    @needs_write_lock
+    def unversion(self, file_ids):
+        """Remove the file ids in file_ids from the current versioned set.
+
+        When a file_id is unversioned, all of its children are automatically
+        unversioned.
+
+        :param file_ids: The file ids to stop versioning.
+        :raises: NoSuchId if any fileid is not currently versioned.
+        """
+        for file_id in file_ids:
+            if self._inventory.has_id(file_id):
+                self._inventory.remove_recursive_id(file_id)
+            else:
+                raise errors.NoSuchId(self, file_id)
+        if len(file_ids):
+            # in the future this should just set a dirty bit to wait for the 
+            # final unlock. However, until all methods of workingtree start
+            # with the current in -memory inventory rather than triggering 
+            # a read, it is more complex - we need to teach read_inventory
+            # to know when to read, and when to not read first... and possibly
+            # to save first when the in memory one may be corrupted.
+            # so for now, we just only write it if it is indeed dirty.
+            # - RBC 20060907
+            self._write_inventory(self._inventory)
+    
     @deprecated_method(zero_eight)
     def iter_conflicts(self):
         """List all files in the tree that have text or content conflicts.
@@ -1333,7 +1358,6 @@
     def kind(self, file_id):
         return file_kind(self.id2abspath(file_id))
 
-    @deprecated_method(zero_eleven)
     def last_revision(self):
         """Return the last revision id of this working tree.
 

